@@ -29,7 +29,7 @@ use strict;
 
 use Bugzilla::RelationSet;
 use vars qw($unconfirmedstate $legal_keywords @legal_platform
-            @legal_priority @legal_severity @legal_opsys @legal_bug_status
+            @legal_priority @legal_severity @legal_opsys @legal_bugs_status
             @settable_resolution %components %versions %target_milestone
             @enterable_products %milestoneurl %prodmaxvotes);
 
@@ -42,11 +42,13 @@ use Bugzilla::Flag;
 use Bugzilla::FlagType;
 use Bugzilla::User;
 use Bugzilla::Util;
+use Bugzilla::Error;
 
 sub fields {
     # Keep this ordering in sync with bugzilla.dtd
     my @fields = qw(bug_id alias creation_ts short_desc delta_ts
                     reporter_accessible cclist_accessible
+                    classification_id classification
                     product component version rep_platform op_sys
                     bug_status resolution
                     bug_file_loc status_whiteboard keywords
@@ -132,22 +134,24 @@ sub initBug  {
      }
   }
 
-  $self->{'whoid'} = $user_id;
+  $self->{'who'} = new Bugzilla::User($user_id);
 
   my $query = "
     SELECT
-      bugs.bug_id, alias, bugs.product_id, products.name, version,
+      bugs.bug_id, alias, products.classification_id, classifications.name,
+      bugs.product_id, products.name, version,
       rep_platform, op_sys, bug_status, resolution, priority,
       bug_severity, bugs.component_id, components.name, assigned_to,
       reporter, bug_file_loc, short_desc, target_milestone,
       qa_contact, status_whiteboard,
       DATE_FORMAT(creation_ts,'%Y.%m.%d %H:%i'),
-      delta_ts, ifnull(sum(votes.vote_count),0),
+      delta_ts, COALESCE(SUM(votes.vote_count), 0),
       reporter_accessible, cclist_accessible,
       estimated_time, remaining_time
     from bugs left join votes using(bug_id),
-      products, components
+      classifications, products, components
     where bugs.bug_id = $bug_id
+      AND classifications.id = products.classification_id
       AND products.id = bugs.product_id
       AND components.id = bugs.component_id
     group by bugs.bug_id";
@@ -155,10 +159,11 @@ sub initBug  {
   &::SendSQL($query);
   my @row = ();
 
-  if ((@row = &::FetchSQLData()) && &::CanSeeBug($bug_id, $self->{'whoid'})) {
+  if ((@row = &::FetchSQLData()) && $self->{'who'}->can_see_bug($bug_id)) {
     my $count = 0;
     my %fields;
-    foreach my $field ("bug_id", "alias", "product_id", "product", "version", 
+    foreach my $field ("bug_id", "alias", "classification_id", "classification",
+                       "product_id", "product", "version", 
                        "rep_platform", "op_sys", "bug_status", "resolution", 
                        "priority", "bug_severity", "component_id", "component",
                        "assigned_to", "reporter", "bug_file_loc", "short_desc",
@@ -265,18 +270,12 @@ sub initBug  {
   return $self;
 }
 
-# Note: If you add a new method, remember that you must check the error
-# state of the bug before returning any data. If $self->{error} is
-# defined, then return something empty. Otherwise you risk potential
-# security holes.
-
 sub dup_id {
     my ($self) = @_;
+
     return $self->{'dup_id'} if exists $self->{'dup_id'};
 
     $self->{'dup_id'} = undef;
-    return if $self->{'error'};
-
     if ($self->{'resolution'} eq 'DUPLICATE') { 
         my $dbh = Bugzilla->dbh;
         $self->{'dup_id'} =
@@ -294,8 +293,7 @@ sub actual_time {
 
     return $self->{'actual_time'} if exists $self->{'actual_time'};
 
-    return undef unless (!$self->{'error'} && Bugzilla->user && 
-                         Bugzilla->user->in_group(Param("timetrackinggroup")));
+    return undef unless Bugzilla->user->in_group(Param("timetrackinggroup"));
 
     my $sth = Bugzilla->dbh->prepare("SELECT SUM(work_time)
                                       FROM longdescs 
@@ -307,9 +305,11 @@ sub actual_time {
 
 sub longdescs {
     my ($self) = @_;
+
     return $self->{'longdescs'} if exists $self->{'longdescs'};
-    return [] if $self->{'error'};
+
     $self->{'longdescs'} = &::GetComments($self->{bug_id});
+
     return $self->{'longdescs'};
 }
 
@@ -319,7 +319,6 @@ sub use_keywords {
 
 sub use_votes {
     my ($self) = @_;
-    return 0 if $self->{'error'};
 
     return Param('usevotes')
       && $::prodmaxvotes{$self->{product}} > 0;
@@ -327,8 +326,8 @@ sub use_votes {
 
 sub groups {
     my $self = shift;
+
     return $self->{'groups'} if exists $self->{'groups'};
-    return [] if $self->{'error'};
 
     my @groups;
 
@@ -392,38 +391,39 @@ sub groups {
 sub user {
     my $self = shift;
     return $self->{'user'} if exists $self->{'user'};
-    return {} if $self->{'error'};
 
-    $self->{'user'} = {};
+    use Bugzilla;
 
-    $self->{'user'}->{'canmove'} = Param('move-enabled')
-                                   && $::userid
-                                   && Bugzilla->user->is_mover;
+    my @movers = map { trim $_ } split(",", Param("movers"));
+    my $canmove = Param("move-enabled") && Bugzilla->user->id && 
+                  (lsearch(\@movers, Bugzilla->user->login) != -1);
 
-    # In the below, if the person hasn't logged in ($::userid == 0), then
-    # we treat them as if they can do anything.  That's because we don't
-    # know why they haven't logged in; it may just be because they don't
-    # use cookies.  Display everything as if they have all the permissions
-    # in the world; their permissions will get checked when they log in
-    # and actually try to make the change.
-    $self->{'user'}->{'canedit'} = $::userid == 0
-                                   || $::userid == $self->{'reporter'}{'id'}
-                                   || (Param('useqacontact') && $self->{'qa_contact'} && $::userid == $self->{'qa_contact'}{'id'})
-                                   || $::userid == $self->{'assigned_to'}{'id'}
-                                   || &::UserInGroup("editbugs");
-    $self->{'user'}->{'canconfirm'} = $::userid == 0
-                                   || ($self->{'qa_contact'} && $::userid == $self->{'qa_contact'}{'id'})
-                                   || $::userid == $self->{'assigned_to'}{'id'}
-                                   || &::UserInGroup("editbugs")
-                                   || &::UserInGroup("canconfirm");
+    # In the below, if the person hasn't logged in, then we treat them
+    # as if they can do anything.  That's because we don't know why they
+    # haven't logged in; it may just be because they don't use cookies.
+    # Display everything as if they have all the permissions in the
+    # world; their permissions will get checked when they log in and
+    # actually try to make the change.
+    my $privileged = (!Bugzilla->user->id)
+                     || Bugzilla->user->in_group("editbugs")
+                     || Bugzilla->user->id == $self->{'assigned_to'}{'id'}
+                     || (Param('useqacontact') && $self->{'qa_contact'} &&
+                         Bugzilla->user->id == $self->{'qa_contact'}{'id'});
+    my $isreporter = Bugzilla->user->id && 
+                     Bugzilla->user->id == $self->{'reporter'}{'id'};
 
+    my $canedit = $privileged || $isreporter;
+    my $canconfirm = $privileged || Bugzilla->user->in_group("canconfirm");
+
+    $self->{'user'} = {canmove    => $canmove, 
+                       canconfirm => $canconfirm, 
+                       canedit    => $canedit,};
     return $self->{'user'};
 }
 
 sub choices {
     my $self = shift;
     return $self->{'choices'} if exists $self->{'choices'};
-    return {} if $self->{'error'};
 
     &::GetVersionTable();
 
@@ -470,7 +470,7 @@ sub choices {
        'priority' => \@::legal_priority,
        'bug_severity' => \@::legal_severity,
        'op_sys' => \@::legal_opsys,
-       'bug_status' => \@::legal_bug_status,
+       'bug_status' => \@::legal_bugs_status,
        'resolution' => \@res,
        'component' => $::components{$self->{product}},
        'version' => $::versions{$self->{product}},
@@ -496,29 +496,10 @@ sub EmitDependList {
 }
 
 sub ValidateTime {
-    my ($time, $field) = @_;
-
-    # regexp verifies one or more digits, optionally followed by a period and
-    # zero or more digits, OR we have a period followed by one or more digits
-    # (allow negatives, though, so people can back out errors in time reporting)
-    if ($time !~ /^-?(?:\d+(?:\.\d*)?|\.\d+)$/) {
-        &::ThrowUserError("number_not_numeric",
-                       {field => "$field", num => "$time"},
-                       "abort");
-    }
-
-    # Only the "work_time" field is allowed to contain a negative value.
-    if ( ($time < 0) && ($field ne "work_time") ) {
-        &::ThrowUserError("number_too_small",
-                       {field => "$field", num => "$time", min_num => "0"},
-                       "abort");
-    }
-
-    if ($time > 99999.99) {
-        &::ThrowUserError("number_too_large",
-                       {field => "$field", num => "$time", max_num => "99999.99"},
-                       "abort");
-    }
+  my ($time, $field) = @_;
+  if ($time > 99999.99 || $time < 0 || !($time =~ /^(?:\d+(?:\.\d*)?|\.\d+)$/)) {
+    ThrowUserError("need_positive_number", {field => "$field"}, "abort");
+  }
 }
 
 sub AUTOLOAD {
