@@ -615,6 +615,15 @@ sub filterEmailGroup ($$$) {
         SendSQL("SELECT emailflags FROM profiles WHERE userid = $userid");
         my $prefs = FetchOneColumn();
         
+        # If the user's preferences are empty, it means the user has not set
+        # their mail preferences after the installation upgraded from a
+        # version of Bugzilla without email preferences to one with them. In
+        # this case, assume they want to receive all mail.
+        if (!defined($prefs) || $prefs !~ /email/) {
+            push(@recipients, $user);
+            next;
+        }
+        
         # Write the user's preferences into a Perl record indexed by 
         # preference name.  We pass the value "255" to the split function 
         # because otherwise split will trim trailing null fields, causing 
@@ -656,7 +665,10 @@ sub filterEmailGroup ($$$) {
         }
         
         # If the user prefers to be included in mail about this change,
-        # add them to the list of recipients.
+        # or they haven't specified a preference for it (because they
+        # haven't visited the email preferences page since the preference
+        # was added, in which case we include them by default), add them
+        # to the list of recipients.
         foreach my $reason (@$reasons) {
             my $pref = "email$role$reason";
             if (!exists($prefs{$pref}) || $prefs{$pref} eq 'on') {
@@ -713,7 +725,7 @@ sub NewProcessOnePerson ($$$$$$$$$$$$$) {
     # see the action of restricting the bug itself; the bug will just 
     # quietly disappear from their radar.
     #
-    return unless CanSeeBug($id, $userid);
+    return unless $user->can_see_bug($id);
 
     #  Drop any non-insiders if the comment is private
     return if (Param("insidergroup") && 
@@ -726,7 +738,7 @@ sub NewProcessOnePerson ($$$$$$$$$$$$$) {
         my $save_id = $dep_id;
         detaint_natural($dep_id) || warn("Unexpected Error: \@depbugs contains a non-numeric value: '$save_id'")
                                  && return;
-        return unless CanSeeBug($dep_id, $userid);
+        return unless $user->can_see_bug($dep_id);
     }
 
     my %mailhead = %defmailhead;
@@ -759,9 +771,9 @@ sub NewProcessOnePerson ($$$$$$$$$$$$$) {
         $add_diff = 0;
         
         if (exists($diff->{'fieldname'}) && 
-         ($diff->{'fieldname'} eq 'estimated_time' ||
-         $diff->{'fieldname'} eq 'remaining_time' ||
-         $diff->{'fieldname'} eq 'work_time')) {
+            ($diff->{'fieldname'} eq 'estimated_time' ||
+             $diff->{'fieldname'} eq 'remaining_time' ||
+             $diff->{'fieldname'} eq 'work_time')) {
             if ($user->groups->{Param("timetrackinggroup")}) {
                 $add_diff = 1;
             }
@@ -850,27 +862,20 @@ sub NewProcessOnePerson ($$$$$$$$$$$$$) {
     
     my $msg = PerformSubsts($template, \%substs);
 
-    MessageToMTA($msg, $person);
-
-    push(@sentlist, $person);
-    return 1;
-}
-
-sub MessageToMTA ($;$) {
-   my ($msg, $rcpt_to) = (@_);
-
-    my $sendmailparam = "";
-    unless (Param("sendmailnow")) {
-       $sendmailparam = "-ODeliveryMode=deferred";
+    my $sendmailparam = "-ODeliveryMode=deferred";
+    if (Param("sendmailnow")) {
+       $sendmailparam = "";
     }
 
     if ($enableSendMail == 1) {
         open(SENDMAIL, "|/usr/lib/sendmail $sendmailparam -t -i") ||
           die "Can't open sendmail";
-
+    
         print SENDMAIL trim($msg) . "\n";
         close SENDMAIL;
     }
+    push(@sentlist, $person);
+    return 1;
 }
 
 1;
