@@ -25,6 +25,7 @@
 #                 J. Paul Reed <preed@sigkill.com>
 #                 Bradley Baetz <bbaetz@student.usyd.edu.au>
 #                 Christopher Aillon <christopher@aillon.com>
+#                 Erik Stambaugh <erik@dasbistro.com>
 
 package Bugzilla::Config;
 
@@ -76,7 +77,7 @@ our $webdotdir = "$datadir/webdot";
 Exporter::export_ok_tags('admin', 'db', 'locations');
 
 # Bugzilla version
-$Bugzilla::Config::VERSION = "2.18.4";
+$Bugzilla::Config::VERSION = "2.19.1";
 
 use Safe;
 
@@ -196,6 +197,19 @@ sub UpdateParams {
     # We don't want it, so get rid of it
     delete $param{'version'};
 
+    # Change from usebrowserinfo to defaultplatform/defaultopsys combo
+    if (exists $param{'usebrowserinfo'}) {
+        if (!$param{'usebrowserinfo'}) {
+            if (!exists $param{'defaultplatform'}) {
+                $param{'defaultplatform'} = 'Other';
+            }
+            if (!exists $param{'defaultopsys'}) {
+                $param{'defaultopsys'} = 'other';
+            }
+        }
+        delete $param{'usebrowserinfo'};
+    }
+
     # Change from a boolean for quips to multi-state
     if (exists $param{'usequip'} && !exists $param{'enablequips'}) {
         $param{'enablequips'} = $param{'usequip'} ? 'on' : 'off';
@@ -217,6 +231,12 @@ sub UpdateParams {
         $param{'loginmethod'} = $param{'useLDAP'} ? "LDAP" : "DB";
     }
 
+    # set verify method to whatever loginmethod was
+    if (exists $param{'loginmethod'} && !exists $param{'user_verify_class'}) {
+        $param{'user_verify_class'} = $param{'loginmethod'};
+        delete $param{'loginmethod'};
+    }
+
     # --- DEFAULTS FOR NEW PARAMS ---
 
     foreach my $item (@param_list) {
@@ -265,8 +285,8 @@ sub WriteParams {
     ChmodDataFile("$datadir/params", 0666);
 }
 
-# Some files in the data directory must be world readable if and only if
-# we don't have a webserver group. Call this function to do this.
+# Some files in the data directory must be world readable iff we don't have
+# a webserver group. Call this function to do this.
 # This will become a private function once all the datafile handling stuff
 # moves into this package
 
