@@ -46,6 +46,18 @@ use base qw(Exporter);
     LOGIN_OPTIONAL
     LOGIN_NORMAL
     LOGIN_REQUIRED
+
+    LOGOUT_ALL
+    LOGOUT_CURRENT
+    LOGOUT_KEEP_CURRENT
+
+    GRANT_DIRECT
+    GRANT_DERIVED
+    GRANT_REGEXP
+
+    GROUP_MEMBERSHIP
+    GROUP_BLESS
+    GROUP_VISIBLE
 );
 
 @Bugzilla::Constants::EXPORT_OK = qw(contenttypes);
@@ -83,7 +95,7 @@ use constant CONTROLMAPSHOWN => 1;
 use constant CONTROLMAPDEFAULT => 2;
 use constant CONTROLMAPMANDATORY => 3;
 
-# See Bugzilla::Auth for docs for these
+# See Bugzilla::Auth for docs on AUTH_*, LOGIN_* and LOGOUT_*
 
 use constant AUTH_OK => 0;
 use constant AUTH_NODATA => 1;
@@ -95,14 +107,27 @@ use constant LOGIN_OPTIONAL => 0;
 use constant LOGIN_NORMAL => 1;
 use constant LOGIN_REQUIRED => 2;
 
+use constant LOGOUT_ALL => 0;
+use constant LOGOUT_CURRENT => 1;
+use constant LOGOUT_KEEP_CURRENT => 2;
+
 use constant contenttypes =>
   {
    "html" => "text/html" , 
-   "rdf" => "application/xml" , 
+   "rdf" => "application/rdf+xml" , 
    "xml" => "text/xml" , 
    "js" => "application/x-javascript" , 
    "csv" => "text/plain" ,
    "png" => "image/png" ,
+   "ics" => "text/calendar" ,
   };
 
+use constant GRANT_DIRECT => 0;
+use constant GRANT_DERIVED => 1;
+use constant GRANT_REGEXP => 2;
+
+use constant GROUP_MEMBERSHIP => 0;
+use constant GROUP_BLESS => 1;
+use constant GROUP_VISIBLE => 2;
+
 1;
