@@ -54,9 +54,6 @@ use Bugzilla::Util;
 # Bugzilla.pm. See bug 192531 for details.
 our $_current_sth;
 our @SQLStateStack = ();
-
-my $_fetchahead;
-
 sub SendSQL {
     my ($str) = @_;
 
@@ -67,8 +64,6 @@ sub SendSQL {
     # This is really really ugly, but its what we get for not doing
     # error checking for 5 years. See bug 189446 and bug 192531
     $_current_sth->{RaiseError} = 0;
-
-    undef $_fetchahead;
 }
 
 # Its much much better to use bound params instead of this
@@ -85,6 +80,8 @@ sub SqlQuote {
     return $res;
 }
 
+# XXX - mod_perl
+my $_fetchahead;
 sub MoreSQLData {
     return 1 if defined $_fetchahead;
 
@@ -159,6 +156,8 @@ sub _connect {
                              FetchHashKeyName => 'NAME',  
                              # Note: NAME_lc causes crash on ActiveState Perl
                              # 5.8.4 (see Bug 253696)
+                             # XXX - This will likely cause problems in DB
+                             # back ends that twiddle column case (Oracle?)
                            });
 
     return $dbh;
@@ -167,9 +166,6 @@ sub _connect {
 sub _handle_error {
     require Carp;
 
-    # Cut down the error string to a reasonable size
-    $_[0] = substr($_[0], 0, 2000) . ' ... ' . substr($_[0], -2000)
-        if length($_[0]) > 4000;
     $_[0] = Carp::longmess($_[0]);
     return 0; # Now let DBI handle raising the error
 }
