@@ -28,6 +28,7 @@ use base qw(Exporter);
 
 use Bugzilla::Config;
 use Bugzilla::Util;
+use Date::Format;
 
 sub _throw_error {
     my ($name, $error, $vars, $unlock_tables) = @_;
@@ -38,6 +39,36 @@ sub _throw_error {
 
     Bugzilla->dbh->do("UNLOCK TABLES") if $unlock_tables;
 
+    # If a writable data/errorlog exists, log error details there.
+    if (-w "data/errorlog") {
+        require Data::Dumper;
+        my $mesg = "";
+        for (1..75) { $mesg .= "-"; };
+        $mesg .= "\n[$$] " . time2str("%D %H:%M:%S ", time());
+        $mesg .= "$name $error ";
+        $mesg .= "$ENV{REMOTE_ADDR} " if $ENV{REMOTE_ADDR};
+        $mesg .= Bugzilla->user->login;
+        $mesg .= "\n";
+        my %params = Bugzilla->cgi->Vars;
+        $Data::Dumper::Useqq = 1;
+        for my $param (sort keys %params) {
+            my $val = $params{$param};
+            # obscure passwords
+            $val = "*****" if $param =~ /password/i;
+            # limit line length
+            $val =~ s/^(.{512}).*$/$1\[CHOP\]/;
+            $mesg .= "[$$] " . Data::Dumper->Dump([$val],["param($param)"]);
+        }
+        for my $var (sort keys %ENV) {
+            my $val = $ENV{$var};
+            $val = "*****" if $val =~ /password|http_pass/i;
+            $mesg .= "[$$] " . Data::Dumper->Dump([$val],["env($var)"]);
+        }
+        open(ERRORLOGFID, ">>data/errorlog");
+        print ERRORLOGFID "$mesg\n";
+        close ERRORLOGFID;
+    }
+
     print Bugzilla->cgi->header();
 
     my $template = Bugzilla->template;
@@ -105,6 +136,10 @@ Bugzilla::Error - Error handling utilities for Bugzilla
 
   ThrowUserError("error_tag",
                  { foo => 'bar' });
+ 
+  # supplying "abort" to ensure tables are unlocked
+  ThrowUserError("another_error_tag",
+                 { foo => 'bar' }, 'abort');
 
 =head1 DESCRIPTION
 
@@ -123,10 +158,11 @@ of variables as a second argument. These are used by the
 I<global/user-error.html.tmpl> template to format the error, using the passed
 in variables as required.
 
-An optional third argument may be supplied. If present (and defined), then the
-error handling code will unlock the database tables. In the long term, this
-argument will go away, to be replaced by transactional C<rollback> calls. There
-is no timeframe for doing so, however.
+An optional third argument may be supplied. If present, the error
+handling code will unlock the database tables: it is a Bugzilla standard
+to provide the string "abort" as the argument value. In the long term,
+this argument will go away, to be replaced by transactional C<rollback>
+calls. There is no timeframe for doing so, however.
 
 =item C<ThrowCodeError>
 
