@@ -36,7 +36,6 @@ use Bugzilla::Config;
 use Bugzilla::Util;
 use Bugzilla::Error;
 use Bugzilla::Attachment;
-use Bugzilla::BugMail;
 
 use constant TABLES_ALREADY_LOCKED => 1;
 
@@ -139,39 +138,17 @@ sub count {
 
 sub validate {
     # Validates fields containing flag modifications.
-    my ($data, $bug_id, $attach_id) = @_;
-    my $dbh = Bugzilla->dbh;
+
+    my ($data, $bug_id) = @_;
   
     # Get a list of flags to validate.  Uses the "map" function
     # to extract flag IDs from form field names by matching fields
     # whose name looks like "flag-nnn", where "nnn" is the ID,
     # and returning just the ID portion of matching field names.
     my @ids = map(/^flag-(\d+)$/ ? $1 : (), keys %$data);
-
-    return unless scalar(@ids);
-
-    # No flag reference should exist when changing several bugs at once.
-    ThrowCodeError("flags_not_available") unless $bug_id;
-
-    # Make sure all flags belong to the bug/attachment they pretend to be.
-    my $field = ($attach_id) ? "attach_id" : "bug_id";
-    my $field_id = $attach_id || $bug_id;
-    my $not = ($attach_id) ? "" : "NOT";
-
-    my $invalid_data =
-        $dbh->selectrow_array("SELECT 1 FROM flags
-                               WHERE id IN (" . join(',', @ids) . ")
-                               AND ($field != ? OR attach_id IS $not NULL)
-                               LIMIT 1",
-                               undef, $field_id);
-
-    if ($invalid_data) {
-        ThrowCodeError("invalid_flag_association",
-                       { bug_id    => $bug_id,
-                         attach_id => $attach_id });
-    }
-
-    foreach my $id (@ids) {
+  
+    foreach my $id (@ids)
+    {
         my $status = $data->{"flag-$id"};
         
         # Make sure the flag exists.
@@ -189,65 +166,49 @@ sub validate {
                             { id => $id, status => $status });
                 
         # Make sure the user didn't request the flag unless it's requestable.
-        # If the flag was requested before it became unrequestable, leave it as is.
-        if ($status eq '?' && $flag->{status} ne '?' && 
-            !$flag->{type}->{is_requestable}) {
+        if ($status eq '?' && !$flag->{type}->{is_requestable}) {
             ThrowCodeError("flag_status_invalid", 
                            { id => $id, status => $status });
         }
-
-        # Make sure the user didn't specify a requestee unless the flag
-        # is specifically requestable. If the requestee was set before
-        # the flag became specifically unrequestable, leave it as is.
-        my $old_requestee =
-            $flag->{'requestee'} ? $flag->{'requestee'}->login : '';
-        my $new_requestee = trim($data->{"requestee-$id"} || '');
-
-        if ($status eq '?'
-            && !$flag->{type}->{is_requesteeble}
-            && $new_requestee
-            && ($new_requestee ne $old_requestee))
-        {
-            ThrowCodeError("flag_requestee_disabled",
-                           { name => $flag->{type}->{name} });
-        }
-
+        
         # Make sure the requestee is authorized to access the bug.
         # (and attachment, if this installation is using the "insider group"
         # feature and the attachment is marked private).
         if ($status eq '?'
             && $flag->{type}->{is_requesteeble}
-            && $new_requestee
-            && ($old_requestee ne $new_requestee))
+            && trim($data->{"requestee-$id"}))
         {
-            # We know the requestee exists because we ran
-            # Bugzilla::User::match_field before getting here.
-            my $requestee = Bugzilla::User->new_from_login($new_requestee);
+            my $requestee_email = trim($data->{"requestee-$id"});
+            if ($requestee_email ne $flag->{'requestee'}->{'email'}) {
+                # We know the requestee exists because we ran
+                # Bugzilla::User::match_field before getting here.
+                my $requestee = Bugzilla::User->new_from_login($requestee_email);
 
-            # Throw an error if the user can't see the bug.
-            if (!&::CanSeeBug($bug_id, $requestee->id))
-            {
-                ThrowUserError("flag_requestee_unauthorized",
-                               { flag_type => $flag->{'type'},
-                                 requestee => $requestee,
-                                 bug_id => $bug_id,
-                                 attach_id =>
-                                   $flag->{target}->{attachment}->{id} });
-            }
+                # Throw an error if the user can't see the bug.
+                if (!$requestee->can_see_bug($bug_id))
+                {
+                    ThrowUserError("flag_requestee_unauthorized",
+                                   { flag_type => $flag->{'type'},
+                                     requestee => $requestee,
+                                     bug_id => $bug_id,
+                                     attach_id =>
+                                       $flag->{target}->{attachment}->{id} });
+                }
 
-            # Throw an error if the target is a private attachment and
-            # the requestee isn't in the group of insiders who can see it.
-            if ($flag->{target}->{attachment}->{exists}
-                && $data->{'isprivate'}
-                && Param("insidergroup")
-                && !$requestee->in_group(Param("insidergroup")))
-            {
-                ThrowUserError("flag_requestee_unauthorized_attachment",
-                               { flag_type => $flag->{'type'},
-                                 requestee => $requestee,
-                                 bug_id    => $bug_id,
-                                 attach_id =>
-                                  $flag->{target}->{attachment}->{id} });
+                # Throw an error if the target is a private attachment and
+                # the requestee isn't in the group of insiders who can see it.
+                if ($flag->{target}->{attachment}->{exists}
+                    && $data->{'isprivate'}
+                    && Param("insidergroup")
+                    && !$requestee->in_group(Param("insidergroup")))
+                {
+                    ThrowUserError("flag_requestee_unauthorized_attachment",
+                                   { flag_type => $flag->{'type'},
+                                     requestee => $requestee,
+                                     bug_id    => $bug_id,
+                                     attach_id =>
+                                      $flag->{target}->{attachment}->{id} });
+                }
             }
         }
     }
@@ -378,16 +339,12 @@ sub migrate {
     # Moves a flag from one attachment to another.  Useful for migrating
     # a flag from an obsolete attachment to the attachment that obsoleted it.
 
-    my ($old_attach_id, $new_attach_id, $timestamp) = @_;
-
-    # Use the date/time we were given if possible (allowing calling code
-    # to synchronize the comment's timestamp with those of other records).
-    $timestamp = ($timestamp ? &::SqlQuote($timestamp) : "NOW()");
+    my ($old_attach_id, $new_attach_id) = @_;
 
     # Update the record in the flags table to point to the new attachment.
     &::SendSQL("UPDATE flags " . 
                "SET    attach_id = $new_attach_id , " . 
-               "       modification_date = $timestamp " . 
+               "       modification_date = NOW() " . 
                "WHERE  attach_id = $old_attach_id");
 }
 
@@ -635,11 +592,11 @@ sub notify {
               || next;
 
             next if $flag->{'target'}->{'bug'}->{'restricted'}
-              && !&::CanSeeBug($flag->{'target'}->{'bug'}->{'id'}, $ccuser->id);
+              && !$ccuser->can_see_bug($flag->{'target'}->{'bug'}->{'id'});
             next if $flag->{'target'}->{'attachment'}->{'isprivate'}
               && Param("insidergroup")
               && !$ccuser->in_group(Param("insidergroup"));
-            push(@new_cc_list, $cc);
+            push(@new_cc_list, $cc.Param('emailsuffix'));
         }
         $flag->{'type'}->{'cc_list'} = join(", ", @new_cc_list);
     }
@@ -655,8 +612,12 @@ sub notify {
         Bugzilla->cgi->header();
         ThrowTemplateError($::template->error());
     }
-
-    Bugzilla::BugMail::MessageToMTA($message);
+    
+    my $delivery_mode = Param("sendmailnow") ? "" : "-ODeliveryMode=deferred";
+    open(SENDMAIL, "|/usr/lib/sendmail $delivery_mode -t -i") 
+      || die "Can't open sendmail";
+    print SENDMAIL $message;
+    close(SENDMAIL);
 }
 
 ################################################################################
