@@ -32,12 +32,11 @@ use strict;
 use vars qw($userid);
 
 package Bugzilla::Search;
-use base qw(Exporter);
-@Bugzilla::Search::EXPORT = qw(IsValidQueryType);
 
 use Bugzilla::Config;
 use Bugzilla::Error;
 use Bugzilla::Util;
+use Bugzilla::Constants;
 
 use Date::Format;
 use Date::Parse;
@@ -97,16 +96,16 @@ sub init {
         push @wherepart, "bugs.product_id = map_products.id";
     }
 
+    if (lsearch($fieldsref, 'map_classifications.name') >= 0) {
+        push @supptables, "classifications AS map_classifications";
+        push @wherepart, "map_products.classification_id = map_classifications.id";
+    }
+
     if (lsearch($fieldsref, 'map_components.name') >= 0) {
         push @supptables, "components AS map_components";
         push @wherepart, "bugs.component_id = map_components.id";
     }
-    
-    if (grep($_ =~ /AS (actual_time|percentage_complete)$/, @$fieldsref)) {
-        push(@supptables, "longdescs AS ldtime");
-        push(@wherepart, "ldtime.bug_id = bugs.bug_id");
-    }
-    
+
     my $minvotes;
     if (defined $params->param('votes')) {
         my $c = trim($params->param('votes'));
@@ -158,7 +157,7 @@ sub init {
     
     my @legal_fields = ("product", "version", "rep_platform", "op_sys",
                         "bug_status", "resolution", "priority", "bug_severity",
-                        "assigned_to", "reporter", "component",
+                        "assigned_to", "reporter", "component", "classification",
                         "target_milestone", "bug_group");
 
     foreach my $field ($params->param()) {
@@ -176,6 +175,11 @@ sub init {
         push(@specialchart, ["keywords", $t, $params->param('keywords')]);
     }
 
+    if (lsearch($fieldsref, "(SUM(ldtime.work_time)*COUNT(DISTINCT ldtime.bug_when)/COUNT(bugs.bug_id)) AS actual_time") != -1) {
+        push(@supptables, "longdescs AS ldtime");
+        push(@wherepart, "ldtime.bug_id = bugs.bug_id");
+    }
+
     foreach my $id ("1", "2") {
         if (!defined ($params->param("email$id"))) {
             next;
@@ -332,6 +336,32 @@ sub init {
     my %funcsbykey;
     my @funcdefs =
         (
+         "^(?:assigned_to|reporter|qa_contact),(?:notequals|equals|anyexact),%group\\.(\\w+)%" => sub {
+             my $group = $1;
+             my $groupid = ValidateGroupName( $group, ($user));
+             $groupid || ThrowUserError('invalid_group_name',{name => $group});
+             my @childgroups = @{$user->flatten_group_membership($groupid)};
+             my $table = "user_group_map_$chartid";
+             push (@supptables, "LEFT JOIN user_group_map $table " .
+                                "ON $table.user_id = bugs.$f " .
+                                "AND $table.group_id IN(" .
+                                join(',', @childgroups) . ") " .
+                                "AND $table.isbless = 0 " .
+                                "AND $table.grant_type IN(" .
+                                GRANT_DIRECT . "," . GRANT_REGEXP . ")"
+                  );
+             if ($t =~ /^not/) {
+                 $term = "$table.group_id IS NULL";
+             } else {
+                 $term = "$table.group_id IS NOT NULL";
+             }
+          },
+         "^(?:assigned_to|reporter|qa_contact),(?:equals|anyexact),(%\\w+%)" => sub {
+             $term = "bugs.$f = " . pronoun($1, $user);
+          },
+         "^(?:assigned_to|reporter|qa_contact),(?:notequals),(%\\w+%)" => sub {
+             $term = "bugs.$f <> " . pronoun($1, $user);
+          },
          "^(assigned_to|reporter)," => sub {
              push(@supptables, "profiles AS map_$f");
              push(@wherepart, "bugs.$f = map_$f.userid");
@@ -340,14 +370,65 @@ sub init {
          "^qa_contact," => sub {
              push(@supptables,
                   "LEFT JOIN profiles map_qa_contact ON bugs.qa_contact = map_qa_contact.userid");
-             $f = "map_$f.login_name";
+             $f = "COALESCE(map_$f.login_name,'')";
          },
 
+         "^(?:cc),(?:notequals|equals|anyexact),%group\\.(\\w+)%" => sub {
+             my $group = $1;
+             my $groupid = ValidateGroupName( $group, ($user));
+             $groupid || ThrowUserError('invalid_group_name',{name => $group});
+             my @childgroups = @{$user->flatten_group_membership($groupid)};
+             my $chartseq = $chartid;
+             if ($chartid eq "") {
+                 $chartseq = "CC$sequence";
+                 $sequence++;
+             }
+             my $table = "user_group_map_$chartseq";
+             push(@supptables, "LEFT JOIN cc cc_$chartseq " .
+                               "ON bugs.bug_id = cc_$chartseq.bug_id");
+             push(@supptables, "LEFT JOIN user_group_map $table " .
+                                "ON $table.user_id = cc_$chartseq.who " .
+                                "AND $table.group_id IN(" .
+                                join(',', @childgroups) . ") " .
+                                "AND $table.isbless = 0 " .
+                                "AND $table.grant_type IN(" .
+                                GRANT_DIRECT . "," . GRANT_REGEXP . ")"
+                  );
+             if ($t =~ /^not/) {
+                 $term = "$table.group_id IS NULL";
+             } else {
+                 $term = "$table.group_id IS NOT NULL";
+             }
+          },
+
+         "^cc,(?:equals|anyexact),(%\\w+%)" => sub {
+             my $match = pronoun($1, $user);
+             my $chartseq = $chartid;
+             if ($chartid eq "") {
+                 $chartseq = "CC$sequence";
+                 $sequence++;
+             }
+             push(@supptables, "LEFT JOIN cc cc_$chartseq 
+                                  ON bugs.bug_id = cc_$chartseq.bug_id 
+                                  AND cc_$chartseq.who = $match");
+             $term = "cc_$chartseq.who IS NOT NULL";
+         },
+         "^cc,(?:notequals),(%\\w+%)" => sub {
+             my $match = pronoun($1, $user);
+             my $chartseq = $chartid;
+             if ($chartid eq "") {
+                 $chartseq = "CC$sequence";
+                 $sequence++;
+             }
+             push(@supptables, "LEFT JOIN cc cc_$chartseq 
+                                  ON bugs.bug_id = cc_$chartseq.bug_id 
+                                  AND cc_$chartseq.who = $match");
+             $term = "cc_$chartseq.who IS NULL";
+         },
          "^cc,(anyexact|substring)" => sub {
              my $list;
              $list = $self->ListIDsForEmail($t, $v);
-             my $chartseq;
-             $chartseq = $chartid;
+             my $chartseq = $chartid;
              if ($chartid eq "") {
                  $chartseq = "CC$sequence";
                  $sequence++;
@@ -365,16 +446,22 @@ sub init {
              }
          },
          "^cc," => sub {
-             my $chartseq;
-             $chartseq = $chartid;
+             my $chartseq = $chartid;
              if ($chartid eq "") {
                  $chartseq = "CC$sequence";
                  $sequence++;
              }
             push(@supptables, "LEFT JOIN cc cc_$chartseq ON bugs.bug_id = cc_$chartseq.bug_id");
 
-            push(@supptables, "LEFT JOIN profiles map_cc_$chartseq ON cc_$chartseq.who = map_cc_$chartseq.userid");
-            $f = "map_cc_$chartseq.login_name";
+            $ff = $f = "map_cc_$chartseq.login_name";
+            my $ref = $funcsbykey{",$t"};
+            &$ref;
+            push(@supptables, 
+                "LEFT JOIN profiles map_cc_$chartseq " .
+                       "ON (cc_$chartseq.who = map_cc_$chartseq.userid " .
+                           "AND ($term))"
+                );
+            $term = "$f IS NOT NULL";
          },
 
          "^long_?desc,changedby" => sub {
@@ -455,11 +542,27 @@ sub init {
          "^content," => sub {
              ThrowUserError("search_content_without_matches");
          },
-         "^commenter," => sub {    
-             my $chartseq;
+         "^commenter,(?:equals|anyexact),(%\\w+%)" => sub {
+             my $match = pronoun($1, $user);
+             my $chartseq = $chartid;
+             if ($chartid eq "") {
+                 $chartseq = "LD$sequence";
+                 $sequence++;
+             }
+             my $table = "longdescs_$chartseq";
+             my $extra = "";
+             if (Param("insidergroup") && !&::UserInGroup(Param("insidergroup"))) {
+                 $extra = "AND $table.isprivate < 1";
+             }
+             push(@supptables, "LEFT JOIN longdescs $table 
+                                  ON $table.bug_id = bugs.bug_id $extra 
+                                  AND $table.who IN ($match)");
+             $term = "$table.who IS NOT NULL";
+         },
+         "^commenter," => sub {
+             my $chartseq = $chartid;
              my $list;
              $list = $self->ListIDsForEmail($t, $v);
-             $chartseq = $chartid;
              if ($chartid eq "") {
                  $chartseq = "LD$sequence";
                  $sequence++;
@@ -626,21 +729,17 @@ sub init {
              }
          },
          "^requestees.login_name," => sub {
-             my $flags = "flags_$chartid";
-             push(@supptables, "LEFT JOIN flags $flags " .
-                               "ON bugs.bug_id = $flags.bug_id " .
-                               "AND $flags.is_active = 1");
-             push(@supptables, "LEFT JOIN profiles requestees_$chartid " .
-                               "ON $flags.requestee_id = requestees_$chartid.userid");
+             push(@supptables, "flags flags_$chartid");
+             push(@wherepart, "bugs.bug_id = flags_$chartid.bug_id");
+             push(@supptables, "profiles requestees_$chartid");
+             push(@wherepart, "flags_$chartid.requestee_id = requestees_$chartid.userid");
              $f = "requestees_$chartid.login_name";
          },
          "^setters.login_name," => sub {
-             my $flags = "flags_$chartid";
-             push(@supptables, "LEFT JOIN flags $flags " .
-                               "ON bugs.bug_id = $flags.bug_id " .
-                               "AND $flags.is_active = 1");
-             push(@supptables, "LEFT JOIN profiles setters_$chartid " .
-                               "ON $flags.setter_id = setters_$chartid.userid");
+             push(@supptables, "flags flags_$chartid");
+             push(@wherepart, "bugs.bug_id = flags_$chartid.bug_id");
+             push(@supptables, "profiles setters_$chartid");
+             push(@wherepart, "flags_$chartid.setter_id = setters_$chartid.userid");
              $f = "setters_$chartid.login_name";
          },
          
@@ -667,6 +766,16 @@ sub init {
                                      $term);
          },
 
+         "^classification,(?!changed)" => sub {
+             # Generate the restriction condition
+             $f = $ff = "classifications.name";
+             $funcsbykey{",$t"}->();
+             $term = build_subselect("map_products.classification_id",
+                                     "classifications.id",
+                                     "classifications",
+                                     $term);
+         },
+
          "^keywords," => sub {
              &::GetVersionTable();
              my @list;
@@ -721,6 +830,12 @@ sub init {
                 push(@wherepart, "$table.dependson = bugs.bug_id");
          },
 
+         "^alias," => sub {
+             $ff = "COALESCE(bugs.alias, '')";
+             my $ref = $funcsbykey{",$t"};
+             &$ref;
+         },
+
          "^owner_idle_time,(greaterthan|lessthan)" => sub {
                 my $table = "idle_" . $chartid;
                 $v =~ /^(\d+)\s*([hHdDwWmMyY])?$/;
@@ -1017,6 +1132,7 @@ sub init {
          $chart < 0 || $params->param("field$chart-0-0") ;
          $chart++) {
         $chartid = $chart >= 0 ? $chart : "";
+        my @chartandlist = ();
         for ($row = 0 ;
              $params->param("field$chart-$row-0") ;
              $row++) {
@@ -1043,13 +1159,15 @@ sub init {
                 # a valid field name, which means that it's ok.
                 trick_taint($f);
                 $q = &::SqlQuote($v);
+                my $rhs = $v;
+                $rhs =~ tr/,//;
                 my $func;
                 $term = undef;
                 foreach my $key (@funcnames) {
-                    if ("$f,$t" =~ m/$key/) {
+                    if ("$f,$t,$rhs" =~ m/$key/) {
                         my $ref = $funcsbykey{$key};
                         if ($debug) {
-                            print "<p>$key ($f , $t ) => ";
+                            print "<p>$key ($f , $t , $rhs ) => ";
                         }
                         $ff = $f;
                         if ($f !~ /\./) {
@@ -1057,7 +1175,7 @@ sub init {
                         }
                         &$ref;
                         if ($debug) {
-                            print "$f , $t , $term</p>";
+                            print "$f , $t , $v , $term</p>";
                         }
                         if ($term) {
                             last;
@@ -1077,7 +1195,14 @@ sub init {
             }
             if (@orlist) {
                 @orlist = map("($_)", @orlist) if (scalar(@orlist) > 1);
-                push(@andlist, "(" . join(" OR ", @orlist) . ")");
+                push(@chartandlist, "(" . join(" OR ", @orlist) . ")");
+            }
+        }
+        if (@chartandlist) {
+            if ($params->param("negate$chart")) {
+                push(@andlist, "NOT(" . join(" AND ", @chartandlist) . ")");
+            } else {
+                push(@andlist, "(" . join(" AND ", @chartandlist) . ")");
             }
         }
     }
@@ -1111,7 +1236,7 @@ sub init {
                 " LEFT JOIN bug_group_map " .
                 " ON bug_group_map.bug_id = bugs.bug_id ";
 
-    if ($user) {
+    if ($user->id) {
         if (%{$user->groups}) {
             $query .= " AND bug_group_map.group_id NOT IN (" . join(',', values(%{$user->groups})) . ") ";
         }
@@ -1120,9 +1245,9 @@ sub init {
     }
 
     $query .= " WHERE " . join(' AND ', (@wherepart, @andlist)) .
-              " AND bugs.creation_ts IS NOT NULL AND ((bug_group_map.group_id IS NULL)";
+              " AND ((bug_group_map.group_id IS NULL)";
 
-    if ($user) {
+    if ($user->id) {
         my $userid = $user->id;
         $query .= "    OR (bugs.reporter_accessible = 1 AND bugs.reporter = $userid) " .
               "    OR (bugs.cclist_accessible = 1 AND cc.who IS NOT NULL) " .
@@ -1274,14 +1399,43 @@ sub getSQL {
     return $self->{'sql'};
 }
 
-# Define if the Query Type passed in is a valid query type that we can deal with
-sub IsValidQueryType
-{
-    my ($queryType) = @_;
-    if (grep { $_ eq $queryType } qw(specific advanced)) {
-        return 1;
+sub pronoun {
+    my ($noun, $user) = (@_);
+    if ($noun eq "%user%") {
+        if ($user->id) {
+            return $user->id;
+        } else {
+            ThrowUserError('login_required_for_pronoun');
+        }
+    }
+    if ($noun eq "%reporter%") {
+        return "bugs.reporter";
+    }
+    if ($noun eq "%assignee%") {
+        return "bugs.assigned_to";
+    }
+    if ($noun eq "%qacontact%") {
+        return "bugs.qa_contact";
     }
     return 0;
 }
 
+# ValidateGroupName checks to see if ANY of the users in the provided list 
+# of user objects can see the named group.  It returns the group id if
+# successful and undef otherwise.
+sub ValidateGroupName {
+    my ($name, @users) = (@_);
+    my @visible = (-1);
+    foreach my $user (@users) {
+        $user && push @visible, @{$user->visible_groups_direct};
+    }
+    my $visible = join(', ', @visible);
+    my $dbh = Bugzilla->dbh;
+    my $sth = $dbh->prepare("SELECT id FROM groups " .
+                            "WHERE name = ? AND id IN($visible)");
+    $sth->execute($name);
+    my ($ret) = $sth->fetchrow_array();
+    return $ret;
+}
+
 1;
