@@ -18,9 +18,10 @@
 # Rights Reserved.
 #
 # Contributor(s): Myk Melez <myk@mozilla.org>
-#                 Erik Stambaugh <not_erik@dasbistro.com>
+#                 Erik Stambaugh <erik@dasbistro.com>
 #                 Bradley Baetz <bbaetz@acm.org>
 #                 Joel Peshkin <bugreport@peshkin.net> 
+#                 Byron Jones <bugzilla@glob.com.au>
 
 ################################################################################
 # Module Initialization
@@ -43,6 +44,9 @@ use Bugzilla::Constants;
 
 sub new {
     my $invocant = shift;
+    if (scalar @_ == 0) {
+        return $invocant->_create;
+    }
     return $invocant->_create("userid=?", @_);
 }
 
@@ -69,6 +73,17 @@ sub _create {
     my $cond = shift;
     my $val = shift;
 
+    # Allow invocation with no parameters to create a blank object
+    my $self = {
+        'id'             => 0,
+        'name'           => '',
+        'login'          => '',
+        'showmybugslink' => 0,
+        'flags'          => {},
+    };
+    bless ($self, $class);
+    return $self unless $cond && $val;
+
     # We're checking for validity here, so any value is OK
     trick_taint($val);
 
@@ -90,13 +105,10 @@ sub _create {
 
     return undef unless defined $id;
 
-    my $self = { id => $id,
-                 name => $name,
-                 login => $login,
-                 showmybugslink => $mybugslink,
-               };
-
-    bless ($self, $class);
+    $self->{'id'}             = $id;
+    $self->{'name'}           = $name;
+    $self->{'login'}          = $login;
+    $self->{'showmybugslink'} = $mybugslink;
 
     # Now update any old group information if needed
     my $result = $dbh->selectrow_array(q{SELECT 1
@@ -128,11 +140,26 @@ sub email { $_[0]->{login} . Param('emailsuffix'); }
 sub name { $_[0]->{name}; }
 sub showmybugslink { $_[0]->{showmybugslink}; }
 
+sub set_flags {
+    my $self = shift;
+    while (my $key = shift) {
+        $self->{'flags'}->{$key} = shift;
+    }
+}
+
+sub get_flag {
+    my $self = shift;
+    my $key = shift;
+    return $self->{'flags'}->{$key};
+}
+
 # Generate a string to identify the user by name + login if the user
 # has a name or by login only if she doesn't.
 sub identity {
     my $self = shift;
 
+    return "" unless $self->id;
+
     if (!defined $self->{identity}) {
         $self->{identity} = 
           $self->{name} ? "$self->{name} <$self->{login}>" : $self->{login};
@@ -144,6 +171,8 @@ sub identity {
 sub nick {
     my $self = shift;
 
+    return "" unless $self->id;
+
     if (!defined $self->{nick}) {
         $self->{nick} = (split(/@/, $self->{login}, 2))[0];
     }
@@ -155,12 +184,17 @@ sub queries {
     my $self = shift;
 
     return $self->{queries} if defined $self->{queries};
+    return [] unless $self->id;
 
     my $dbh = Bugzilla->dbh;
-    my $sth = $dbh->prepare(q{  SELECT name, query, linkinfooter
-                                  FROM namedqueries
-                                 WHERE userid=?
-                              ORDER BY UPPER(name)});
+    my $sth = $dbh->prepare(q{ SELECT
+                             DISTINCT name, query, linkinfooter,
+                                      IF(whine_queries.id IS NOT NULL, 1, 0)
+                                 FROM namedqueries
+                            LEFT JOIN whine_queries
+                                   ON whine_queries.query_name = name
+                                WHERE userid=?
+                             ORDER BY UPPER(name)});
     $sth->execute($self->{id});
 
     my @queries;
@@ -169,6 +203,7 @@ sub queries {
                           name         => $row->[0],
                           query        => $row->[1],
                           linkinfooter => $row->[2],
+                          usedinwhine  => $row->[3],
                         });
     }
     $self->{queries} = \@queries;
@@ -186,6 +221,7 @@ sub groups {
     my $self = shift;
 
     return $self->{groups} if defined $self->{groups};
+    return {} unless $self->id;
 
     my $dbh = Bugzilla->dbh;
     my $groups = $dbh->selectcol_arrayref(q{SELECT DISTINCT groups.name, group_id
@@ -209,6 +245,7 @@ sub in_group {
 
     # If we already have the info, just return it.
     return defined($self->{groups}->{$group}) if defined $self->{groups};
+    return 0 unless $self->id;
 
     # Otherwise, go check for it
 
@@ -227,10 +264,116 @@ sub in_group {
     return defined($res);
 }
 
+sub can_see_bug {
+    my ($self, $bugid) = @_;
+    my $dbh = Bugzilla->dbh;
+    my $sth  = $self->{sthCanSeeBug};
+    my $userid  = $self->{id};
+    # Get fields from bug, presence of user on cclist, and determine if
+    # the user is missing any groups required by the bug. The prepared query
+    # is cached because this may be called for every row in buglists or
+    # every bug in a dependency list.
+    unless ($sth) {
+        $sth = $dbh->prepare("SELECT reporter, assigned_to, qa_contact,
+                             reporter_accessible, cclist_accessible,
+                             COUNT(cc.who), COUNT(bug_group_map.bug_id)
+                             FROM bugs
+                             LEFT JOIN cc 
+                               ON cc.bug_id = bugs.bug_id
+                               AND cc.who = $userid
+                             LEFT JOIN bug_group_map 
+                               ON bugs.bug_id = bug_group_map.bug_id
+                               AND bug_group_map.group_ID NOT IN(" .
+                               join(',',(-1, values(%{$self->groups}))) .
+                               ") WHERE bugs.bug_id = ? GROUP BY bugs.bug_id");
+    }
+    $sth->execute($bugid);
+    my ($reporter, $owner, $qacontact, $reporter_access, $cclist_access,
+        $isoncclist, $missinggroup) = $sth->fetchrow_array();
+    $self->{sthCanSeeBug} = $sth;
+    return ( (($reporter == $userid) && $reporter_access)
+           || (Param('useqacontact') && ($qacontact == $userid) && $userid)
+           || ($owner == $userid)
+           || ($isoncclist && $cclist_access)
+           || (!$missinggroup) );
+}
+
+sub get_selectable_products {
+    my ($self, $by_id) = @_;
+
+    if (defined $self->{SelectableProducts}) {
+        my %list = @{$self->{SelectableProducts}};
+        return \%list if $by_id;
+        return values(%list);
+    }
+
+    my $query = "SELECT id, name " .
+                "FROM products " .
+                "LEFT JOIN group_control_map " .
+                "ON group_control_map.product_id = products.id ";
+    if (Param('useentrygroupdefault')) {
+        $query .= "AND group_control_map.entry != 0 ";
+    } else {
+        $query .= "AND group_control_map.membercontrol = " .
+                  CONTROLMAPMANDATORY . " ";
+    }
+    $query .= "AND group_id NOT IN(" . 
+               join(',', (-1,values(%{Bugzilla->user->groups}))) . ") " .
+              "WHERE group_id IS NULL ORDER BY name";
+    my $dbh = Bugzilla->dbh;
+    my $sth = $dbh->prepare($query);
+    $sth->execute();
+    my @products = ();
+    while (my @row = $sth->fetchrow_array) {
+        push(@products, @row);
+    }
+    $self->{SelectableProducts} = \@products;
+    my %list = @products;
+    return \%list if $by_id;
+    return values(%list);
+}
+
+# visible_groups_inherited returns a reference to a list of all the groups
+# whose members are visible to this user.
+sub visible_groups_inherited {
+    my $self = shift;
+    return $self->{visible_groups_inherited} if defined $self->{visible_groups_inherited};
+    return [] unless $self->id;
+    my @visgroups = @{$self->visible_groups_direct};
+    @visgroups = @{$self->flatten_group_membership(@visgroups)};
+    $self->{visible_groups_inherited} = \@visgroups;
+    return $self->{visible_groups_inherited};
+}
+
+# visible_groups_direct returns a reference to a list of all the groups that
+# are visible to this user.
+sub visible_groups_direct {
+    my $self = shift;
+    my @visgroups = ();
+    return $self->{visible_groups_direct} if defined $self->{visible_groups_direct};
+    return [] unless $self->id;
+
+    my $dbh = Bugzilla->dbh;
+    my $glist = join(',',(-1,values(%{$self->groups})));
+    my $sth = $dbh->prepare("SELECT DISTINCT grantor_id
+                                FROM group_group_map
+                               WHERE member_id IN($glist)
+                                 AND grant_type=" . GROUP_VISIBLE);
+    $sth->execute();
+
+    while (my ($row) = $sth->fetchrow_array) {
+        push @visgroups,$row;
+    }
+    $self->{visible_groups_direct} = \@visgroups;
+
+    return $self->{visible_groups_direct};
+}
+
 sub derive_groups {
     my ($self, $already_locked) = @_;
 
     my $id = $self->id;
+    return unless $id;
 
     my $dbh = Bugzilla->dbh;
 
@@ -287,9 +430,10 @@ sub derive_groups {
             $group_sth ||= $dbh->prepare(q{SELECT grantor_id
                                              FROM group_group_map
                                             WHERE member_id=?
-                                              AND isbless=0});
+                                              AND grant_type=' . 
+                                                  GROUP_MEMBERSHIP . '});
             $group_sth->execute($group);
-            while (my $groupid = $group_sth->fetchrow_array) {
+            while (my ($groupid) = $group_sth->fetchrow_array) {
                 if (!defined($groupidschecked{"$groupid"})) {
                     push(@groupidstocheck,$groupid);
                 }
@@ -317,6 +461,7 @@ sub can_bless {
     my $self = shift;
 
     return $self->{can_bless} if defined $self->{can_bless};
+    return 0 unless $self->id;
 
     my $dbh = Bugzilla->dbh;
     # First check if the user can explicitly bless a group
@@ -332,7 +477,8 @@ sub can_bless {
                                             FROM user_group_map, group_group_map
                                            WHERE user_group_map.user_id=?
                                              AND user_group_map.group_id=member_id
-                                             AND group_group_map.isbless=1},
+                                             AND group_group_map.grant_type=} .
+                                                 GROUP_BLESS,
                                         undef,
                                         $self->{id});
     }
@@ -342,6 +488,30 @@ sub can_bless {
     return $self->{can_bless};
 }
 
+sub flatten_group_membership {
+    my ($self, @groups) = @_;
+
+    my $dbh = Bugzilla->dbh;
+    my $sth;
+    my @groupidstocheck = @groups;
+    my %groupidschecked = ();
+    $sth = $dbh->prepare("SELECT member_id FROM group_group_map
+                             WHERE grantor_id = ? 
+                               AND grant_type = " . GROUP_MEMBERSHIP);
+    while (my $node = shift @groupidstocheck) {
+        $sth->execute($node);
+        my $member;
+        while (($member) = $sth->fetchrow_array) {
+            if (!$groupidschecked{$member}) {
+                $groupidschecked{$member} = 1;
+                push @groupidstocheck, $member;
+                push @groups, $member unless grep $_ == $member, @groups;
+            }
+        }
+    }
+    return \@groups;
+}
+
 sub match {
     # Generates a list of users whose login name (email address) or real name
     # matches a substring or wildcard.
@@ -364,17 +534,28 @@ sub match {
     # first try wildcards
 
     my $wildstr = $str;
+    my $user = Bugzilla->user;
 
     if ($wildstr =~ s/\*/\%/g && # don't do wildcards if no '*' in the string
         Param('usermatchmode') ne 'off') { # or if we only want exact matches
 
         # Build the query.
         my $sqlstr = &::SqlQuote($wildstr);
-        my $query  = "SELECT userid, realname, login_name " .
-                     "FROM profiles " .
-                     "WHERE (login_name LIKE $sqlstr " .
+        my $query  = "SELECT DISTINCT userid, realname, login_name " .
+                     "FROM profiles ";
+        if (&::Param('usevisibilitygroups')) {
+            $query .= ", user_group_map ";
+        }
+        $query    .= "WHERE (login_name LIKE $sqlstr " .
                      "OR realname LIKE $sqlstr) ";
-        $query    .= "AND disabledtext = '' " if $exclude_disabled;
+        if (&::Param('usevisibilitygroups')) {
+            $query .= "AND user_group_map.user_id = userid " .
+                      "AND isbless = 0 " .
+                      "AND group_id IN(" .
+                      join(', ', (-1, @{$user->visible_groups_inherited})) . ") " .
+                      "AND grant_type <> " . GRANT_DERIVED;
+        }
+        $query    .= " AND disabledtext = '' " if $exclude_disabled;
         $query    .= "ORDER BY length(login_name) ";
         $query    .= "LIMIT $limit " if $limit;
 
@@ -410,14 +591,23 @@ sub match {
 
         my $sqlstr = &::SqlQuote(uc($str));
 
-        my $query  = "SELECT  userid, realname, login_name " .
-                     "FROM  profiles " .
-                     "WHERE  (INSTR(UPPER(login_name), $sqlstr) " .
-                     "OR INSTR(UPPER(realname), $sqlstr)) ";
-        $query    .= "AND disabledtext = '' " if $exclude_disabled;
+        my $query  = "SELECT DISTINCT userid, realname, login_name " .
+                     "FROM  profiles ";
+        if (&::Param('usevisibilitygroups')) {
+            $query .= ", user_group_map ";
+        }
+        $query     .= "WHERE  (INSTR(UPPER(login_name), $sqlstr) " .
+                      "OR INSTR(UPPER(realname), $sqlstr)) ";
+        if (&::Param('usevisibilitygroups')) {
+            $query .= "AND user_group_map.user_id = userid " .
+                      "AND isbless = 0 " .
+                      "AND group_id IN(" .
+                      join(', ', (-1, @{$user->visible_groups_inherited})) . ") " .
+                      "AND grant_type <> " . GRANT_DERIVED;
+        }
+        $query    .= " AND disabledtext = '' " if $exclude_disabled;
         $query    .= "ORDER BY length(login_name) ";
         $query    .= "LIMIT $limit " if $limit;
-
         &::PushGlobalSQLState();
         &::SendSQL($query);
         push(@users, new Bugzilla::User(&::FetchSQLData())) while &::MoreSQLData();
@@ -649,6 +839,7 @@ sub email_prefs {
     # Get or set (not implemented) the user's email notification preferences.
     
     my $self = shift;
+    return {} unless $self->id;
     
     # If the calling code is setting the email preferences, update the object
     # but don't do anything else.  This needs to write email preferences back
@@ -666,6 +857,20 @@ sub email_prefs {
     my @reasons = qw(Removeme Comments Attachments Status Resolved Keywords 
                      CC Other Unconfirmed);
 
+    # If the prefs are empty, this user hasn't visited the email pane
+    # of userprefs.cgi since before the change to use the "emailflags" 
+    # column, so initialize that field with the default prefs.
+    if (!$flags) {
+        # Create a default prefs string that causes the user to get all email.
+        $flags = "ExcludeSelf~on~FlagRequestee~on~FlagRequester~on~";
+        foreach my $role (@roles) {
+            foreach my $reason (@reasons) {
+                $flags .= "email$role$reason~on~";
+            }
+        }
+        chop $flags;
+    }
+
     # Convert the prefs from the flags string from the database into
     # a Perl record.  The 255 param is here because split will trim 
     # any trailing null fields without a third param, which causes Perl 
@@ -698,15 +903,43 @@ sub email_prefs {
     return $self->{email_prefs};
 }
 
-sub is_mover {
+sub get_userlist {
     my $self = shift;
 
-    if (!defined $self->{'is_mover'}) {
-        my @movers = map { trim($_) } split(',', Param('movers'));
-        $self->{'is_mover'} = ($self->id
-                               && lsearch(\@movers, $self->login) != -1);
+    return $self->{'userlist'} if defined $self->{'userlist'};
+
+    my $query  = "SELECT DISTINCT login_name, realname,";
+    if (&::Param('usevisibilitygroups')) {
+        $query .= " COUNT(group_id) ";
+    } else {
+        $query .= " 1 ";
     }
-    return $self->{'is_mover'};
+        $query .= "FROM profiles ";
+    if (&::Param('usevisibilitygroups')) {
+        $query .= "LEFT JOIN user_group_map " .
+                  "ON user_group_map.user_id = userid AND isbless = 0 " .
+                  "AND group_id IN(" .
+                  join(', ', (-1, @{$self->visible_groups_inherited})) . ") " .
+                  "AND grant_type <> " . GRANT_DERIVED;
+    }
+    $query    .= " WHERE disabledtext = '' GROUP BY userid";
+
+    my $dbh = Bugzilla->dbh;
+    my $sth = $dbh->prepare($query);
+    $sth->execute;
+
+    my @userlist;
+    while (my($login, $name, $visible) = $sth->fetchrow_array) {
+        push @userlist, {
+            login => $login,
+            identity => $name ? "$name <$login>" : $login,
+            visible => $visible,
+        };
+    }
+    @userlist = sort { lc $$a{'identity'} cmp lc $$b{'identity'} } @userlist;
+
+    $self->{'userlist'} = \@userlist;
+    return $self->{'userlist'};
 }
 
 1;
@@ -737,8 +970,10 @@ L<Bugzilla-E<gt>user|Bugzilla/"user">.
 
 =item C<new($userid)>
 
-Creates a new C<Bugzilla::User> object for the given user id. Returns
-C<undef> if no matching user is found.
+Creates a new C{Bugzilla::User> object for the given user id.  If no user
+id was given, a blank object is created with no user attributes.
+
+If an id was given but there was no matching user found, undef is returned.
 
 =begin undocumented
 
@@ -832,6 +1067,10 @@ intended for cases where we are not looking at the currently logged in user,
 and only need to make a quick check for the group, where calling C<groups>
 and getting all of the groups would be overkill.
 
+=item C<can_see_bug(bug_id)>
+
+Determines if the user can see the specified bug.
+
 =item C<derive_groups>
 
 Bugzilla allows for group inheritance. When data about the user (or any of the
@@ -840,10 +1079,41 @@ care of by the constructor. However, when updating the email address, the
 user may be placed into different groups, based on a new email regexp. This
 method should be called in such a case to force reresolution of these groups.
 
+=item C<get_selectable_products(by_id)>
+
+Returns an alphabetical list of product names from which
+the user can select bugs.  If the $by_id parameter is true, it returns
+a hash where the keys are the product ids and the values are the
+product names.
+
+=item C<get_userlist>
+
+Returns a reference to an array of users.  The array is populated with hashrefs
+containing the login, identity and visibility.  Users that are not visible to this
+user will have 'visible' set to zero.
+
+=item C<flatten_group_membership>
+
+Accepts a list of groups and returns a list of all the groups whose members 
+inherit membership in any group on the list.  So, we can determine if a user
+is in any of the groups input to flatten_group_membership by querying the
+user_group_map for any user with DIRECT or REGEXP membership IN() the list
+of groups returned.
+
+=item C<visible_groups_inherited>
+
+Returns a list of all groups whose members should be visible to this user.
+Since this list is flattened already, there is no need for all users to
+be have derived groups up-to-date to select the users meeting this criteria.
+
+=item C<visible_groups_direct>
+
+Returns a list of groups that the user is aware of.
+
 =begin undocumented
 
 This routine takes an optional argument. If true, then this routine will not
-lock the tables, but will rely on the caller to ahve done so itsself.
+lock the tables, but will rely on the caller to have done so itsself.
 
 This is required because mysql will only execute a query if all of the tables
 are locked, or if none of them are, not a mixture. If the caller has already
@@ -861,11 +1131,20 @@ all MySQL supported, this will go away.
 
 Returns C<1> if the user can bless at least one group. Otherwise returns C<0>.
 
-=item C<is_mover>
+=item C<set_flags>
+=item C<get_flag>
 
-Returns true if the user is in the list of users allowed to move bugs
-to another database. Note that this method doesn't check whether bug
-moving is enabled.
+User flags are template-accessible user status information, stored in the form
+of a hash.  For an example of use, when the current user is authenticated in
+such a way that they are allowed to log out, the 'can_logout' flag is set to
+true (1).  The template then checks this flag before displaying the "Log Out"
+link.
+
+C<set_flags> is called with any number of key,value pairs.  Flags for each key
+will be set to the specified value.
+
+C<get_flag> is called with a single key name, which returns the associated
+value.
 
 =back
 
