@@ -44,7 +44,6 @@ use Bugzilla::Util;
 use Bugzilla::Config;
 use Bugzilla::Constants;
 use Bugzilla::Error;
-use Bugzilla::BugMail;
 
 # Shut up misguided -w warnings about "used only once".  For some reason,
 # "use vars" chokes on me when I try it here.
@@ -61,21 +60,15 @@ require 'globals.pl';
 use vars qw($template $vars);
 
 # If Bugzilla is shut down, do not go any further, just display a message
-# to the user about the downtime and log out.  (do)editparams.cgi is exempted
-# from this message, of course, since it needs to be available in order for
+# to the user about the downtime.  (do)editparams.cgi is exempted from
+# this message, of course, since it needs to be available in order for
 # the administrator to open Bugzilla back up.
 if (Param("shutdownhtml") && $0 !~ m:(^|[\\/])(do)?editparams\.cgi$:) {
-    # For security reasons, log out users when Bugzilla is down.
-    # Bugzilla->login() is required to catch the logincookie, if any.
-    my $user = Bugzilla->login(LOGIN_OPTIONAL);
-    my $userid = defined($user) ? $user->id : 0;
-    Bugzilla->logout();
+    $::vars->{'message'} = "shutdown";
     
     # Return the appropriate HTTP response headers.
     print Bugzilla->cgi->header();
     
-    $::vars->{'message'} = "shutdown";
-    $::vars->{'userid'} = $userid;
     # Generate and return an HTML message about the downtime.
     $::template->process("global/message.html.tmpl", $::vars)
       || ThrowTemplateError($::template->error());
@@ -155,7 +148,7 @@ sub ValidateBugID {
     # database, and that the user is authorized to access that bug.
     # We detaint the number here, too
 
-    my ($id, $field) = @_;
+    my ($id, $skip_authorization) = @_;
     
     # Get rid of white-space around the ID.
     $id = trim($id);
@@ -164,9 +157,7 @@ sub ValidateBugID {
     my $alias = $id;
     if (!detaint_natural($id)) {
         $id = BugAliasToID($alias);
-        $id || ThrowUserError("invalid_bug_id_or_alias",
-                              {'bug_id' => $alias,
-                               'field'  => $field });
+        $id || ThrowUserError("invalid_bug_id_or_alias", {'bug_id' => $alias});
     }
     
     # Modify the calling code's original variable to contain the trimmed,
@@ -179,9 +170,9 @@ sub ValidateBugID {
     FetchOneColumn()
       || ThrowUserError("invalid_bug_id_non_existent", {'bug_id' => $id});
 
-    return if (defined $field && ($field eq "dependson" || $field eq "blocked"));
+    return if $skip_authorization;
     
-    return if CanSeeBug($id, $::userid);
+    return if Bugzilla->user->can_see_bug($id);
 
     # The user did not pass any of the authorization tests, which means they
     # are not authorized to see the bug.  Display an error and stop execution.
@@ -232,7 +223,9 @@ sub MailPassword {
                              "login" => $login,
                              "password" => $password});
 
-    Bugzilla::BugMail::MessageToMTA($msg, $login . Param('emailsuffix'));
+    open SENDMAIL, "|/usr/lib/sendmail -t -i";
+    print SENDMAIL $msg;
+    close SENDMAIL;
 }
 
 sub PutHeader {
@@ -250,13 +243,11 @@ sub PutFooter {
 
 sub CheckIfVotedConfirmed {
     my ($id, $who) = (@_);
-    PushGlobalSQLState();
     SendSQL("SELECT bugs.votes, bugs.bug_status, products.votestoconfirm, " .
             "       bugs.everconfirmed " .
             "FROM bugs, products " .
             "WHERE bugs.bug_id = $id AND products.id = bugs.product_id");
     my ($votes, $status, $votestoconfirm, $everconfirmed) = (FetchSQLData());
-    my $ret = 0;
     if ($votes >= $votestoconfirm && $status eq $::unconfirmedstate) {
         SendSQL("UPDATE bugs SET bug_status = 'NEW', everconfirmed = 1 " .
                 "WHERE bug_id = $id");
@@ -280,15 +271,12 @@ sub CheckIfVotedConfirmed {
         
         $template->process("bug/process/results.html.tmpl", $vars)
           || ThrowTemplateError($template->error());
-        $ret = 1;
     }
-    PopGlobalSQLState();
-    return $ret;
+
 }
 sub LogActivityEntry {
     my ($i,$col,$removed,$added,$whoid,$timestamp) = @_;
-    # in the case of CCs, deps, and keywords, there's a possibility that someone
-    # might try to add or remove a lot of them at once, which might take more
+    # in the case of CCs, deps, and keywords, there's a possibility that someone    # might try to add or remove a lot of them at once, which might take more
     # space than the activity table allows.  We'll solve this by splitting it
     # into multiple entries if it's too long.
     while ($removed || $added) {
@@ -335,10 +323,10 @@ sub GetBugActivity {
         $suppwhere = "AND NOT(COALESCE(attachments.isprivate,0))"; 
     }
     my $query = "
-        SELECT IFNULL(fielddefs.description, bugs_activity.fieldid),
+        SELECT COALESCE(fielddefs.description, bugs_activity.fieldid),
                 fielddefs.name,
                 bugs_activity.attach_id,
-                DATE_FORMAT(bugs_activity.bug_when,'%Y.%m.%d %H:%i:%s'),
+                DATE_FORMAT(bugs_activity.bug_when,'%Y.%m.%d %H:%i'),
                 bugs_activity.removed, bugs_activity.added,
                 profiles.login_name
         FROM bugs_activity $suppjoins LEFT JOIN fielddefs ON 
@@ -439,10 +427,6 @@ foreach my $name ($::cgi->param()) {
 
 $::buffer = $::cgi->query_string();
 
-foreach my $name ($::cgi->cookie()) {
-    $::COOKIE{$name} = $::cgi->cookie($name);
-}
-
 # This could be needed in any CGI, so we set it here.
 $vars->{'help'} = $::cgi->param('help') ? 1 : 0;
 
