@@ -25,6 +25,7 @@
 #                 J. Paul Reed <preed@sigkill.com>
 #                 Bradley Baetz <bbaetz@student.usyd.edu.au>
 #                 Joseph Heenan <joseph@heenan.me.uk>
+#                 Erik Stambaugh <erik@dasbistro.com>
 #
 
 # This file defines all the parameters that we have a GUI to edit within
@@ -65,6 +66,36 @@ sub check_priority {
     return "";
 }
 
+sub check_severity {
+    my ($value) = (@_);
+    &::GetVersionTable();
+    if (lsearch(\@::legal_severity, $value) < 0) {
+        return "Must be a legal severity value: one of " .
+            join(", ", @::legal_severity);
+    }
+    return "";
+}
+
+sub check_platform {
+    my ($value) = (@_);
+    &::GetVersionTable();
+    if (lsearch(['', @::legal_platform], $value) < 0) {
+        return "Must be empty or a legal platform value: one of " .
+            join(", ", @::legal_platform);
+    }
+    return "";
+}
+
+sub check_opsys {
+    my ($value) = (@_);
+    &::GetVersionTable();
+    if (lsearch(['', @::legal_opsys], $value) < 0) {
+        return "Must be empty or a legal operating system value: one of " .
+            join(", ", @::legal_opsys);
+    }
+    return "";
+}
+
 sub check_shadowdb {
     my ($value) = (@_);
     $value = trim($value);
@@ -127,7 +158,7 @@ sub check_netmask {
     return "";
 }
 
-sub check_loginmethod {
+sub check_user_verify_class {
     # doeditparams traverses the list of params, and for each one it checks,
     # then updates. This means that if one param checker wants to look at 
     # other params, it must be below that other one. So you can't have two 
@@ -136,18 +167,20 @@ sub check_loginmethod {
     # the login method as LDAP, we won't notice, but all logins will fail.
     # So don't do that.
 
-    my ($method, $entry) = @_;
-    my $res = check_multi($method, $entry);
-    return $res if $res;
-    if ($method eq 'DB') {
-        # No params
-    } elsif ($method eq 'LDAP') {
-        eval "require Net::LDAP";
-        return "Error requiring Net::LDAP: '$@'" if $@;
-        return "LDAP servername is missing" unless Param("LDAPserver");
-        return "LDAPBaseDN is empty" unless Param("LDAPBaseDN");
-    } else {
-        return "Unknown loginmethod '$method' in check_loginmethod";
+    my ($list, $entry) = @_;
+    for my $class (split /,\s*/, $list) {
+        my $res = check_multi($class, $entry);
+        return $res if $res;
+        if ($class eq 'DB') {
+            # No params
+        } elsif ($class eq 'LDAP') {
+            eval "require Net::LDAP";
+            return "Error requiring Net::LDAP: '$@'" if $@;
+            return "LDAP servername is missing" unless Param("LDAPserver");
+            return "LDAPBaseDN is empty" unless Param("LDAPBaseDN");
+        } else {
+                return "Unknown user_verify_class '$class' in check_user_verify_class";
+        }
     }
     return "";
 }
@@ -317,6 +350,23 @@ sub find_languages {
    checker => \&check_multi
   },
 
+  {
+   name => 'useclassification',
+   desc => 'If this is on, Bugzilla will associate each product with a ' .
+           'specific classification.  But you must have "editclassification" ' .
+           'permissions enabled in order to edit classifications',
+   type => 'b',
+   default => 0
+  },
+
+  {
+   name => 'showallproducts',
+   desc => 'If this is on and useclassification is set, Bugzilla will add a' .
+           '"All" link in the "New Bug" page to list all available products',
+   type => 'b',
+   default => 0
+  },
+
   {
    name => 'makeproductgroups',
    desc => 'If this is on, Bugzilla will associate a bug group with each ' .
@@ -433,8 +483,74 @@ sub find_languages {
   },
 
   {
-   name => 'loginmethod',
-   desc => 'The type of login authentication to use:
+   name => 'auth_env_id',
+   desc    => 'Environment variable used by external authentication system ' .
+              'to store a unique identifier for each user.  Leave it blank ' .
+              'if there isn\'t one or if this method of authentication ' .
+              'is not being used.',
+   type    => 't',
+   default => '',
+  },
+
+  {
+   name    => 'auth_env_email',
+   desc    => 'Environment variable used by external authentication system ' .
+              'to store each user\'s email address.  This is a required ' .
+              'field for environmental authentication.  Leave it blank ' .
+              'if you are not going to use this feature.',
+   type    => 't',
+   default => '',
+  },
+
+  {
+   name    => 'auth_env_realname',
+   desc    => 'Environment variable used by external authentication system ' .
+              'to store the user\'s real name.  Leave it blank if there ' .
+              'isn\'t one or if this method of authentication is not being ' .
+              'used.',
+   type    => 't',
+   default => '',
+  },
+
+  # XXX in the future:
+  #
+  # user_verify_class and user_info_class should have choices gathered from
+  # whatever sits in their respective directories
+  #
+  # rather than comma-separated lists, these two should eventually become
+  # arrays, but that requires alterations to editparams first
+
+  {
+   name => 'user_info_class',
+   desc => 'Mechanism(s) to be used for gathering a user\'s login information.
+              <add>
+            More than one may be selected. If the first one returns nothing,
+            the second is tried, and so on.<br />
+            The types are:
+            <dl>
+              <dt>CGI</dt>
+              <dd>
+                Asks for username and password via CGI form interface.
+              </dd>
+              <dt>Env</dt>
+              <dd>
+                Info for a pre-authenticated user is passed in system
+                environment variables.
+              </dd>
+            </dl>',
+   type => 's',
+   choices => [ 'CGI', 'Env', 'Env,CGI' ],
+   default => 'CGI',
+   checker => \&check_multi
+  },
+
+  {
+   name => 'user_verify_class',
+   desc => 'Mechanism(s) to be used for verifying (authenticating) information
+            gathered by user_info_class.
+            More than one may be selected. If the first one cannot find the
+            user, the second is tried, and so on.<br />
+            The types are:
             <dl>
               <dt>DB</dt>
               <dd>
@@ -450,9 +566,9 @@ sub find_languages {
               </dd>
              </dl>',
    type => 's',
-   choices => [ 'DB', 'LDAP' ],
+   choices => [ 'DB', 'LDAP', 'DB,LDAP', 'LDAP,DB' ],
    default => 'DB',
-   checker => \&check_loginmethod
+   checker => \&check_user_verify_class
   },
 
   {
@@ -503,13 +619,11 @@ sub find_languages {
 
   {
    name => 'sendmailnow',
-   desc => 'Sites using anything older than version 8.12 of \'sendmail\' ' .
-           'can achieve a significant performance increase in the ' .
-           'UI -- at the cost of delaying the sending of mail -- by ' .
-           'disabling this parameter. Sites using \'sendmail\' 8.12 or ' .
-           'higher should leave this on, as they will see no benefit from ' .
-           'turning it off. Sites using an MTA other than \'sendmail\' ' .
-           '*must* leave it on, or no bug mail will be sent.',
+   desc => 'If this is on, Bugzilla will tell sendmail to send any e-mail ' .
+           'immediately. If you have a large number of users with a large ' .
+           'amount of e-mail traffic, enabling this option may dramatically ' .
+           'slow down Bugzilla. Best used for smaller installations of ' .
+           'Bugzilla.',
    type => 'b',
    default => 1
   },
@@ -570,9 +684,8 @@ Configure bugmail: %urlbase%userprefs.cgi?tab=email
 
   {
    name => 'whinedays',
-   desc => q{The number of days that we'll let a bug sit untouched in a NEW
-             state before our cronjob will whine at the owner.<br>
-             Set to 0 to disable whining.},
+   desc => 'The number of days that we\'ll let a bug sit untouched in a NEW ' .
+           'state before our cronjob will whine at the owner.',
    type => 't',
    default => 7
   },
@@ -634,8 +747,8 @@ You will get this message once a day until you\'ve dealt with these bugs!
   {
    name => 'defaultquery',
    desc => 'This is the default query that initially comes up when you ' .
-           'access the advanced query page.  It\'s in URL parameter ' .
-           'format, which makes it hard to read.  Sorry!',
+           'submit a bug.  It\'s in URL parameter format, which makes it ' .
+           'hard to read.  Sorry!',
    type => 't',
    default => 'bug_status=NEW&bug_status=ASSIGNED&bug_status=REOPENED&emailassigned_to1=1&emailassigned_to2=1&emailreporter2=1&emailcc2=1&emailqa_contact2=1&order=Importance&long_desc_type=substring'
   },
@@ -657,6 +770,38 @@ You will get this message once a day until you\'ve dealt with these bugs!
    checker => \&check_priority
   },
 
+  {
+   name => 'defaultseverity',
+   desc => 'This is the severity that newly entered bugs are set to.',
+   type => 't',
+   default => 'normal',
+   checker => \&check_severity
+  },
+
+  {
+    name => 'defaultplatform',
+    desc => 'This is the platform that is preselected on the bug '.
+            'entry form.<br>'.
+            'You can leave this empty: '.
+            'Bugzilla will then use the platform that the browser '.
+            'reports to be running on as the default.',
+    type => 't',
+    default => '',
+    checker => \&check_platform
+  },
+
+  {
+    name => 'defaultopsys',
+    desc => 'This is the operating system that is preselected on the bug '.
+            'entry form.<br>'.
+            'You can leave this empty: '.
+            'Bugzilla will then use the operating system that the browser '.
+            'reports to be running on as the default.',
+    type => 't',
+    default => '',
+    checker => \&check_opsys
+  },
+
   {
    name => 'usetargetmilestone',
    desc => 'Do you wish to use the Target Milestone field?',
@@ -686,14 +831,6 @@ You will get this message once a day until you\'ve dealt with these bugs!
    default => 0
   },
 
-  {
-   name => 'usebrowserinfo',
-   desc => 'Do you want bug reports to be assigned an OS & Platform based ' .
-           'on the browser the user makes the report from?',
-   type => 'b',
-   default => 1
-  },
-
   {
    name => 'usevotes',
    desc => 'Do you wish to allow users to vote for bugs? Note that in order ' .
@@ -712,6 +849,14 @@ You will get this message once a day until you\'ve dealt with these bugs!
    default => 0
   },
 
+  {
+   name => 'usevisibilitygroups',
+   desc => 'Do you wish to restrict visibility of users to members of ' .
+           'specific groups?',
+   type => 'b',
+   default => 0
+  },
+
   {
    name => 'webdotbase',
    desc => 'It is possible to show graphs of dependent bugs. You may set ' .
@@ -1084,6 +1229,15 @@ Reason: %reason%
    default => '0'
   },
 
+  {
+   name => 'usemenuforusers',
+   desc => 'If this option is set, a popup menu will be used where a user' .
+           ' needs to be selected.  This option should not be enabled on' .
+           ' sites where there are a large number of users.',
+   type => 'b',
+   default => '0'
+  },
+
   {
    name => 'usermatchmode',
    desc => 'Allow match strings to be entered for user names when entering ' .
