@@ -33,7 +33,6 @@ use Bugzilla::Constants;
 use Bugzilla::Util;
 # Bring ChmodDataFile in until this is all moved to the module
 use Bugzilla::Config qw(:DEFAULT ChmodDataFile $localconfig $datadir);
-use Bugzilla::BugMail;
 
 # Shut up misguided -w warnings about "used only once".  For some reason,
 # "use vars" chokes on me when I try it here.
@@ -56,6 +55,7 @@ sub globals_pl_sillyness {
     $zz = @main::legal_versions;
     $zz = @main::milestoneurl;
     $zz = %main::proddesc;
+    $zz = %main::classdesc;
     $zz = @main::prodmaxvotes;
     $zz = $main::template;
     $zz = $main::userid;
@@ -116,19 +116,23 @@ $::unconfirmedstate = "UNCONFIRMED";
 sub AppendComment {
     my ($bugid, $who, $comment, $isprivate, $timestamp, $work_time) = @_;
     $work_time ||= 0;
-
-    if ($work_time) {
-        require Bugzilla::Bug;
-        Bugzilla::Bug::ValidateTime($work_time, "work_time");
-    }
-
+    
     # Use the date/time we were given if possible (allowing calling code
     # to synchronize the comment's timestamp with those of other records).
     $timestamp = ($timestamp ? SqlQuote($timestamp) : "NOW()");
-
+    
     $comment =~ s/\r\n/\n/g;     # Get rid of windows-style line endings.
     $comment =~ s/\r/\n/g;       # Get rid of mac-style line endings.
 
+    # allowing negatives though so people can back out errors in time reporting
+    if (defined $work_time) {
+       # regexp verifies one or more digits, optionally followed by a period and
+       # zero or more digits, OR we have a period followed by one or more digits
+       if ($work_time !~ /^-?(?:\d+(?:\.\d*)?|\.\d+)$/) { 
+          ThrowUserError("need_numeric_value", {}, "abort");
+       }
+    } else { $work_time = 0 };
+
     if ($comment =~ /^\s*$/) {  # Nothin' but whitespace
         return;
     }
@@ -181,6 +185,19 @@ sub GenerateVersionTable {
         $carray{$c} = 1;
     }
 
+    SendSQL("SELECT products.name, classifications.name " .
+            "FROM products, classifications " .
+            "WHERE classifications.id = products.classification_id " .
+            "ORDER BY classifications.name");
+    while (@line = FetchSQLData()) {
+        my ($p,$c) = (@line);
+        if (!defined $::classifications{$c}) {
+            $::classifications{$c} = [];
+        }
+        my $ref = $::classifications{$c};
+        push @$ref, $p;
+    }
+
     my $dotargetmilestone = 1;  # This used to check the param, but there's
                                 # enough code that wants to pretend we're using
                                 # target milestones, even if they don't get
@@ -188,6 +205,13 @@ sub GenerateVersionTable {
                                 # about them anyway.
 
     my $mpart = $dotargetmilestone ? ", milestoneurl" : "";
+
+    SendSQL("select name, description from classifications ORDER BY name");
+    while (@line = FetchSQLData()) {
+        my ($n, $d) = (@line);
+        $::classdesc{$n} = $d;
+    }
+
     SendSQL("select name, description, votesperuser, disallownew$mpart from products ORDER BY name");
     while (@line = FetchSQLData()) {
         my ($p, $d, $votesperuser, $dis, $u) = (@line);
@@ -272,8 +296,10 @@ sub GenerateVersionTable {
                                    '*::legal_bug_status', '*::legal_resolution']));
 
     print $fh (Data::Dumper->Dump([\@::settable_resolution, \%::proddesc,
+                                   \%::classifications, \%::classdesc,
                                    \@::enterable_products, \%::prodmaxvotes],
                                   ['*::settable_resolution', '*::proddesc',
+                                   '*::classifications', '*::classdesc',
                                    '*::enterable_products', '*::prodmaxvotes']));
 
     if ($dotargetmilestone) {
@@ -408,15 +434,13 @@ sub InsertNewUser {
     my $cryptpassword = Crypt($password);
 
 
-    my $defaultflagstring = SqlQuote(Bugzilla::Constants::DEFAULT_EMAIL_SETTINGS); 
-
     # Insert the new user record into the database.            
     $username = SqlQuote($username);
     $realname = SqlQuote($realname);
     $cryptpassword = SqlQuote($cryptpassword);
     PushGlobalSQLState();
-    SendSQL("INSERT INTO profiles (login_name, realname, cryptpassword, emailflags) 
-             VALUES ($username, $realname, $cryptpassword, $defaultflagstring)");
+    SendSQL("INSERT INTO profiles (login_name, realname, cryptpassword) 
+             VALUES ($username, $realname, $cryptpassword)");
     PopGlobalSQLState();
 
     # Return the password to the calling code so it can be included 
@@ -481,7 +505,7 @@ sub CanEditProductId {
     my $query = "SELECT group_id FROM group_control_map " .
                 "WHERE product_id = $productid " .
                 "AND canedit != 0 "; 
-    if (defined Bugzilla->user && %{Bugzilla->user->groups}) {
+    if (%{Bugzilla->user->groups}) {
         $query .= "AND group_id NOT IN(" . 
                    join(',', values(%{Bugzilla->user->groups})) . ") ";
     }
@@ -493,70 +517,44 @@ sub CanEditProductId {
     return (!defined($result));
 }
 
-# This function determines whether or not a user can enter
-# bugs into the named product.
+sub IsInClassification {
+    my ($classification,$productname) = @_;
+
+    if (! Param('useclassification')) {
+        return 1;
+    } else {
+        my $query = "SELECT classifications.name " .
+          "FROM products,classifications " .
+            "WHERE products.classification_id=classifications.id ";
+        $query .= "AND products.name = " . SqlQuote($productname);
+        PushGlobalSQLState();
+        SendSQL($query);
+        my ($ret) = FetchSQLData();
+        PopGlobalSQLState();
+        return ($ret eq $classification);
+    }
+}
+
+#
+# This function determines if a user can enter bugs in the named
+# product.
 sub CanEnterProduct {
-    my ($productname, $verbose) = @_;
-    my $dbh = Bugzilla->dbh;
-
-    return unless defined($productname);
-    trick_taint($productname);
-
-    # First check whether or not the user has access to that product.
+    my ($productname) = @_;
     my $query = "SELECT group_id IS NULL " .
                 "FROM products " .
                 "LEFT JOIN group_control_map " .
                 "ON group_control_map.product_id = products.id " .
                 "AND group_control_map.entry != 0 ";
-    if (defined Bugzilla->user && %{Bugzilla->user->groups}) {
+    if (%{Bugzilla->user->groups}) {
         $query .= "AND group_id NOT IN(" . 
                    join(',', values(%{Bugzilla->user->groups})) . ") ";
     }
-    $query .= "WHERE products.name = ? LIMIT 1";
-
-    my $has_access = $dbh->selectrow_array($query, undef, $productname);
-    if (!$has_access) {
-        # Do we require the exact reason why we cannot enter
-        # bugs into that product? Returning -1 explicitely
-        # means the user has no access to the product or the
-        # product does not exist.
-        return (defined($verbose)) ? -1 : 0;
-    }
-
-    # Check if the product is open for new bugs and has
-    # at least one component.
-    my $allow_new_bugs =
-        $dbh->selectrow_array("SELECT CASE WHEN disallownew = 0 THEN 1 ELSE 0 END
-                               FROM products INNER JOIN components
-                               ON components.product_id = products.id
-                               WHERE products.name = ? LIMIT 1",
-                               undef, $productname);
-
-    # Return 1 if the user can enter bugs into that product;
-    # return 0 if the product is closed for new bug entry;
-    # return undef if the product has no component.
-    return $allow_new_bugs;
-}
-
-# Call CanEnterProduct() and display an error message
-# if the user cannot enter bugs into that product.
-sub CanEnterProductOrWarn {
-    my ($product) = @_;
-
-    if (!defined($product)) {
-        ThrowUserError("no_products");
-    }
-    my $status = CanEnterProduct($product, 1);
-    trick_taint($product);
-
-    if (!defined($status)) {
-        ThrowUserError("no_components", { product => $product});
-    } elsif (!$status) {
-        ThrowUserError("product_disabled", { product => $product});
-    } elsif ($status < 0) {
-        ThrowUserError("entry_access_denied", { product => $product});
-    }
-    return $status;
+    $query .= "WHERE products.name = " . SqlQuote($productname) . " LIMIT 1";
+    PushGlobalSQLState();
+    SendSQL($query);
+    my ($ret) = FetchSQLData();
+    PopGlobalSQLState();
+    return ($ret);
 }
 
 sub GetEnterableProducts {
@@ -570,18 +568,21 @@ sub GetEnterableProducts {
     return (@products);
 }
 
+
 #
 # This function returns an alphabetical list of product names to which
 # the user can enter bugs.  If the $by_id parameter is true, also retrieves IDs
 # and pushes them onto the list as id, name [, id, name...] for easy slurping
 # into a hash by the calling code.
 sub GetSelectableProducts {
-    my ($by_id) = @_;
+    my ($by_id,$by_classification) = @_;
 
     my $extra_sql = $by_id ? "id, " : "";
 
-    my $query = "SELECT $extra_sql name " .
-                "FROM products " .
+    my $extra_from_sql = $by_classification ? ", classifications" : "";
+
+    my $query = "SELECT $extra_sql products.name " .
+                "FROM products $extra_from_sql " .
                 "LEFT JOIN group_control_map " .
                 "ON group_control_map.product_id = products.id ";
     if (Param('useentrygroupdefault')) {
@@ -590,11 +591,17 @@ sub GetSelectableProducts {
         $query .= "AND group_control_map.membercontrol = " .
                   CONTROLMAPMANDATORY . " ";
     }
-    if (defined Bugzilla->user && %{Bugzilla->user->groups}) {
+    if (%{Bugzilla->user->groups}) {
         $query .= "AND group_id NOT IN(" . 
                    join(',', values(%{Bugzilla->user->groups})) . ") ";
     }
-    $query .= "WHERE group_id IS NULL ORDER BY name";
+    $query .= "WHERE group_id IS NULL ";
+    if ($by_classification) {
+        $query .= "AND classifications.id = products.classification_id ";
+        $query .= "AND classifications.name = ";
+        $query .= SqlQuote($by_classification) . " ";
+    }
+    $query .= "ORDER BY name";
     PushGlobalSQLState();
     SendSQL($query);
     my @products = ();
@@ -627,23 +634,21 @@ sub GetSelectableProductHash {
 
     PushGlobalSQLState();
     foreach my $table (@tables) {
+        # Why oh why can't we standardize on these names?!?
+        my $fld = ($table eq "components" ? "name" : "value");
+
+        my $query = "SELECT $fld, product_id FROM $table WHERE product_id IN " .
+                    "(" . join(",", keys %products) . ") ORDER BY $fld";
+        SendSQL($query);
+
         my %values;
         my %values_by_product;
 
-        if (scalar(keys %products)) {
-            # Why oh why can't we standardize on these names?!?
-            my $fld = ($table eq "components" ? "name" : "value");
-
-            my $query = "SELECT $fld, product_id FROM $table WHERE product_id " .
-                        "IN (" . join(",", keys %products) . ") ORDER BY $fld";
-            SendSQL($query);
-
-            while (MoreSQLData()) {
-                my ($name, $product_id) = FetchSQLData();
-                next unless $name;
-                $values{$name} = 1;
-                push @{$values_by_product{$products{$product_id}}}, $name;
-            }
+        while (MoreSQLData()) {
+            my ($name, $product_id) = FetchSQLData();
+            next unless $name;
+            $values{$name} = 1;
+            push @{$values_by_product{$products{$product_id}}}, $name;
         }
 
         $selectables->{"legal_$table"} = [sort keys %values];
@@ -654,18 +659,29 @@ sub GetSelectableProductHash {
     return $selectables;
 }
 
+#
+# This function returns an alphabetical list of classifications that has products the user can enter bugs.
+sub GetSelectableClassifications {
+    my @selectable_classes = ();
+
+    foreach my $c (keys %::classdesc) {
+        if ( scalar(GetSelectableProducts(0,$c)) > 0) {
+           push(@selectable_classes,$c);
+        }
+    }
+    return (@selectable_classes);
+}
 
 sub GetFieldDefs {
     my $extra = "";
     if (!UserInGroup(Param('timetrackinggroup'))) {
-        $extra = "AND name NOT IN ('estimated_time', 'remaining_time', " .
+        $extra = "WHERE name NOT IN ('estimated time', 'remaining_time', " .
                  "'work_time', 'percentage_complete')";
     }
 
     my @fields;
     PushGlobalSQLState();
-    SendSQL("SELECT name, description FROM fielddefs " .
-            "WHERE obsolete = 0 $extra ORDER BY sortkey");
+    SendSQL("SELECT name, description FROM fielddefs $extra ORDER BY sortkey");
     while (MoreSQLData()) {
         my ($name, $description) = FetchSQLData();
         push(@fields, { name => $name, description => $description });
@@ -676,51 +692,6 @@ sub GetFieldDefs {
 }
 
 
-sub CanSeeBug {
-
-    my ($id, $userid) = @_;
-
-    # Query the database for the bug, retrieving a boolean value that
-    # represents whether or not the user is authorized to access the bug.
-
-    # if no groups are found --> user is permitted to access
-    # if no user is found for any group --> user is not permitted to access
-    my $query = "SELECT bugs.bug_id, reporter, assigned_to, qa_contact," .
-        " reporter_accessible, cclist_accessible," .
-        " cc.who IS NOT NULL," .
-        " COUNT(DISTINCT(bug_group_map.group_id)) as cntbugingroups," .
-        " COUNT(DISTINCT(user_group_map.group_id)) as cntuseringroups" .
-        " FROM bugs" .
-        " LEFT JOIN cc ON bugs.bug_id = cc.bug_id" .
-        " AND cc.who = $userid" .
-        " LEFT JOIN bug_group_map ON bugs.bug_id = bug_group_map.bug_id" .
-        " LEFT JOIN user_group_map ON" .
-        " user_group_map.group_id = bug_group_map.group_id" .
-        " AND user_group_map.isbless = 0" .
-        " AND user_group_map.user_id = $userid" .
-        " WHERE bugs.bug_id = $id AND creation_ts IS NOT NULL" .
-        " GROUP BY bugs.bug_id";
-    PushGlobalSQLState();
-    SendSQL($query);
-    my ($found_id, $reporter, $assigned_to, $qa_contact,
-        $rep_access, $cc_access,
-        $found_cc, $found_groups, $found_members) 
-        = FetchSQLData();
-    PopGlobalSQLState();
-    return ($found_id
-            &&
-              (($found_groups == 0) 
-               || (($userid > 0) && 
-                  (
-                       ($assigned_to == $userid) 
-                    || (Param('useqacontact') && $qa_contact == $userid)
-                    || (($reporter == $userid) && $rep_access) 
-                    || ($found_cc && $cc_access) 
-                    || ($found_groups == $found_members)
-                  ))
-              )
-           );
-}
 
 sub ValidatePassword {
     # Determines whether or not a password is valid (i.e. meets Bugzilla's
@@ -730,11 +701,11 @@ sub ValidatePassword {
     my ($password, $matchpassword) = @_;
     
     if (length($password) < 3) {
-        ThrowUserError("password_too_short", $::vars);
+        ThrowUserError("password_too_short");
     } elsif (length($password) > 16) {
-        ThrowUserError("password_too_long", $::vars);
+        ThrowUserError("password_too_long");
     } elsif ((defined $matchpassword) && ($password ne $matchpassword)) {
-        ThrowUserError("passwords_dont_match", $::vars);
+        ThrowUserError("passwords_dont_match");
     }
 }
 
@@ -831,6 +802,25 @@ sub DBNameToIdAndCheck {
                    { name => $name }, "abort");
 }
 
+sub get_classification_id {
+    my ($classification) = @_;
+    PushGlobalSQLState();
+    SendSQL("SELECT id FROM classifications WHERE name = " . SqlQuote($classification));
+    my ($classification_id) = FetchSQLData();
+    PopGlobalSQLState();
+    return $classification_id;
+}
+
+sub get_classification_name {
+    my ($classification_id) = @_;
+    die "non-numeric classification_id '$classification_id' passed to get_classification_name"
+      unless ($classification_id =~ /^\d+$/);
+    PushGlobalSQLState();
+    SendSQL("SELECT name FROM classifications WHERE id = $classification_id");
+    my ($classification) = FetchSQLData();
+    PopGlobalSQLState();
+    return $classification;
+}
 
 
 
@@ -996,7 +986,7 @@ sub GetAttachmentLink {
             my ($bugid, $isobsolete, $desc) = FetchSQLData();
             my $title = "";
             my $className = "";
-            if (CanSeeBug($bugid, $::userid)) {
+            if (Bugzilla->user->can_see_bug($bugid)) {
                 $title = $desc;
             }
             if ($isobsolete) {
@@ -1036,11 +1026,8 @@ sub GetAttachmentLink {
 
 sub GetBugLink {
     my ($bug_num, $link_text, $comment_num) = @_;
-    if (! defined $bug_num || $bug_num eq "") {
-        return "&lt;missing bug number&gt;";
-    }
-    my $quote_bug_num = html_quote($bug_num);
-    detaint_natural($bug_num) || return "&lt;invalid bug number: $quote_bug_num&gt;";
+    $bug_num || return "&lt;missing bug number&gt;";
+    detaint_natural($bug_num) || return "&lt;invalid bug number&gt;";
 
     # If we've run GetBugLink() for this bug number before, %::buglink
     # will contain an anonymous array ref of relevent values, if not
@@ -1070,7 +1057,7 @@ sub GetBugLink {
                 $title .= " $bug_res";
                 $post = '</span>';
             }
-            if (CanSeeBug($bug_num, $::userid)) {
+            if (Bugzilla->user->can_see_bug($bug_num)) {
                 $title .= " - $bug_desc";
             }
             $::buglink{$bug_num} = [$pre, value_quote($title), $post];
@@ -1244,7 +1231,7 @@ sub UserInGroup {
         die "UserInGroup no longer takes a second parameter.";
     }
     
-    return defined Bugzilla->user && defined Bugzilla->user->groups->{$_[0]};
+    return defined Bugzilla->user->groups->{$_[0]};
 }
 
 sub UserCanBlessGroup {
@@ -1269,7 +1256,7 @@ sub UserCanBlessGroup {
         WHERE groups.id = grantor_id 
         AND user_group_map.user_id = $::userid
         AND user_group_map.isbless = 0
-        AND group_group_map.isbless = 1
+        AND group_group_map.grant_type = " . GROUP_BLESS . "
         AND user_group_map.group_id = member_id
         AND groups.name = " . SqlQuote($groupname));
     $result = FetchOneColumn();
@@ -1421,26 +1408,32 @@ sub RemoveVotes {
 
             # Now lets send the e-mail to alert the user to the fact that their votes have
             # been reduced or removed.
-            my %substs;
+            my $sendmailparm = '-ODeliveryMode=deferred';
+            if (Param('sendmailnow')) {
+               $sendmailparm = '';
+            }
+            if (open(SENDMAIL, "|/usr/lib/sendmail $sendmailparm -t -i")) {
+                my %substs;
 
-            $substs{"to"} = $name . Param('emailsuffix');
-            $substs{"bugid"} = $id;
-            $substs{"reason"} = $reason;
+                $substs{"to"} = $name;
+                $substs{"bugid"} = $id;
+                $substs{"reason"} = $reason;
 
-            $substs{"votesremoved"} = $removedvotes;
-            $substs{"votesold"} = $oldvotes;
-            $substs{"votesnew"} = $newvotes;
+                $substs{"votesremoved"} = $removedvotes;
+                $substs{"votesold"} = $oldvotes;
+                $substs{"votesnew"} = $newvotes;
 
-            $substs{"votesremovedtext"} = $removedvotestext;
-            $substs{"votesoldtext"} = $oldvotestext;
-            $substs{"votesnewtext"} = $newvotestext;
+                $substs{"votesremovedtext"} = $removedvotestext;
+                $substs{"votesoldtext"} = $oldvotestext;
+                $substs{"votesnewtext"} = $newvotestext;
 
-            $substs{"count"} = $removedvotes . "\n    " . $newvotestext;
+                $substs{"count"} = $removedvotes . "\n    " . $newvotestext;
 
-            my $msg = PerformSubsts(Param("voteremovedmail"),
-                                    \%substs);
-
-            Bugzilla::BugMail::MessageToMTA($msg, $substs{'to'});
+                my $msg = PerformSubsts(Param("voteremovedmail"),
+                                        \%substs);
+                print SENDMAIL $msg;
+                close SENDMAIL;
+            }
         }
         SendSQL("SELECT SUM(vote_count) FROM votes WHERE bug_id = $id");
         my $v = FetchOneColumn();
