@@ -23,20 +23,21 @@
 
 package Support::Files;
 
-use File::Find;
-
 # exclude_deps is a hash of arrays listing the files to be excluded
 # if a module is not available
 #
 @additional_files = ();
 %exclude_deps = (
     'XML::Parser' => ['importxml.pl'],
-    'Net::LDAP' => ['Bugzilla/Auth/LDAP.pm'],
+    'Net::LDAP' => ['Bugzilla/Auth/Verify/LDAP.pm'],
 );
 
 
-@files = glob('*');
-find(sub { push(@files, $File::Find::name) if $_ =~ /\.pm$/;}, 'Bugzilla');
+# XXX - this file should really be rewritten to use File::Find or similar
+$file = '*';
+@files = (glob($file), glob('Bugzilla/*.pm'), glob('Bugzilla/*/*.pm'),
+          glob('Bugzilla/*/*/*.pm'), glob('Bugzilla/*/*/*/*.pm'),
+          glob('Bugzilla/*/*/*/*/*.pm'));
 
 sub have_pkg {
     my ($pkg) = @_;
