@@ -30,13 +30,27 @@
 # Values always used for numbers  - [% (i|j|k|n|count) %]
 # Params                          - [% Param(...
 # Safe functions                  - [% (time2str|GetBugLink)...
-# Safe vmethods                   - [% foo.size %]
+# Safe vmethods                   - [% foo.size %] [% foo.length %]
+#                                   [% foo.push() %]
 # TT loop variables               - [% loop.count %]
 # Already-filtered stuff          - [% wibble FILTER html %]
 #   where the filter is one of html|csv|js|url_quote|quoteUrls|time|uri|xml|none
 
 %::safe = (
 
+'whine/schedule.html.tmpl' => [
+  'event.key',
+  'query.id',
+  'query.sort',
+  'schedule.id',
+  'option.0',
+  'option.1',
+],
+
+'whine/mail.html.tmpl' => [
+  'bug.bug_id',
+],
+
 'sidebar.xul.tmpl' => [
   'template_version', 
 ],
@@ -101,7 +115,6 @@
                                                             IF sortvisible',
   'column.name', 
   'column.description',
-  'vis_bug_ids.push(bug.id)', 
   'bug.id', 
   'bug.count', 
   'bug.delta', 
@@ -239,6 +252,10 @@
   'VERSION', 
 ],
 
+'global/choose-classification.html.tmpl' => [
+  'classdesc.$p', 
+],
+
 'global/choose-product.html.tmpl' => [
   'target',
   'proddesc.$p', 
@@ -253,7 +270,6 @@
 'global/header.html.tmpl' => [
   'javascript', 
   'style', 
-  'style_url', 
   'bgcolor', 
   'onload',
   'bodyattrs', 
@@ -478,11 +494,33 @@
   'section_num'
 ],
 
+'admin/table.html.tmpl' => [
+  'link_uri',
+  'c.content'
+],
+
+'admin/classifications/del.html.tmpl' => [
+  'description', 
+],
+
+'admin/classifications/edit.html.tmpl' => [
+  'description', 
+],
+
+'admin/classifications/reclassify.html.tmpl' => [
+  'description', 
+],
+
+'admin/classifications/select.html.tmpl' => [
+  'cl.description', 
+],
+
 'admin/products/groupcontrol/confirm-edit.html.tmpl' => [
   'group.count', 
 ],
 
 'admin/products/groupcontrol/edit.html.tmpl' => [
+  'filt_classification', 
   'filt_product', 
   'group.bugcount', 
   'group.id', 
@@ -492,11 +530,6 @@
   'const.CONTROLMAPMANDATORY', 
 ],
 
-'admin/keywords/list.html.tmpl' => [
-  'keyword.id',
-  'keyword.bug_count',
-],
-
 'admin/keywords/edit.html.tmpl' => [
   'keyword_id',
   'bug_count',
@@ -526,6 +559,35 @@
   'type.flag_count', 
 ],
 
+
+'admin/components/confirm-delete.html.tmpl' => [
+  'bug_count'
+],
+
+'admin/components/deleted.html.tmpl' => [
+  'deleted_bug_count'
+],
+
+'admin/components/edit.html.tmpl' => [
+  'bug_count'
+],
+
+'admin/components/list.html.tmpl' => [
+  'cgi.query_string'
+],
+
+'admin/components/select-product.html.tmpl' => [
+  'cgi.query_string'
+],
+
+'admin/milestones/confirm-delete.html.tmpl' => [
+  'bug_count'
+],
+
+'admin/milestones/deleted.html.tmpl' => [
+  'deleted_bug_count'
+],
+
 'account/login.html.tmpl' => [
   'target', 
 ],
