@@ -28,15 +28,9 @@
 # touched for more than the number of days specified in the whinedays param.
 
 use strict;
-use lib '.';
 
 require "globals.pl";
 
-use Bugzilla::BugMail;
-
-# Whining is disabled if whinedays is zero
-exit unless Param('whinedays') >= 1;
-
 SendSQL("select bug_id,short_desc,login_name from bugs,profiles where " .
         "(bug_status = 'NEW' or bug_status = 'REOPENED') and " . 
         "to_days(now()) - to_days(delta_ts) > " . Param('whinedays') .
@@ -74,7 +68,10 @@ foreach my $email (sort (keys %bugs)) {
         $msg .= "    -> ${urlbase}show_bug.cgi?id=$i\n";
     }
 
-    Bugzilla::BugMail::MessageToMTA($msg, $substs{'email'});
-
+    my $sendmailparam = Param('sendmailnow') ? '' : "-ODeliveryMode=deferred";
+    open SENDMAIL, "|/usr/lib/sendmail $sendmailparam -t -i"
+        or die "Can't open sendmail";
+    print SENDMAIL $msg;
+    close SENDMAIL;
     print "$email      " . join(" ", @{$bugs{$email}}) . "\n";
 }
